/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxAnimSet.h
 * @date 13/02/2004
 * @author iarwain@orx-project.org
 *
 * @todo
 * - Add multi-channel support (events, custom, ...)
 '/

/'*
 * @addtogroup orxAnimSet
 *
 * Animation (Set) Module.
 * Allows to creates and handle Sets of Anims.
 * It consists of a structure containing Anims and their relations.
 * It also contains functions for handling and accessing them.
 * AnimSets are structures.
 * They thus can be referenced by Anim Pointers.
 *
 * @{
 '/


#ifndef _orxANIMSET_BI
#define _orxANIMSET_BI


#include "orxInclude.bi"
#include "orx/anim/orxAnim.bi"


/'* AnimSet flags
 '/
#define orxANIMSET_KU32_FLAG_NONE                   &H00000000  /'*< No flags. '/

#define orxANIMSET_KU32_FLAG_REFERENCE_LOCK         &H00010000  /'*< If there's already a reference on it, the AnimSet is locked for changes. '/
#define orxANIMSET_KU32_FLAG_LINK_STATIC            &H00020000  /'*< If not static, animpointer should duplicate the link table upon linking/creation. '/

/'* AnimSet Link Flags
 '/
#define orxANIMSET_KU32_LINK_FLAG_NONE              &H00000000  /'*< No flags. '/

#define orxANIMSET_KU32_LINK_FLAG_LOOP_COUNTER      &H10000000  /'*< Anim link uses a counter '/
#define orxANIMSET_KU32_LINK_FLAG_PRIORITY          &H20000000  /'*< Anim link has priority '/
#define orxANIMSET_KU32_LINK_FLAG_IMMEDIATE_CUT     &H40000000  /'*< Anim link should update with an immediate cut '/
#define orxANIMSET_KU32_LINK_FLAG_CLEAR_TARGET      &H80000000  /'*< Anim link should clear target when used '/

/'* AnimSet defines
 '/
#define orxANIMSET_KU32_MAX_ANIM_NUMBER             128         /'*< Maximum number of Anims for an Anim set structure '/


/'* Internal AnimSet structure
 '/
Type __orxANIMSET_t As Any
Type orxANIMSET As __orxANIMSET_t

/'* Internal Link Table structure
 '/
Type __orxANIMSET_LINK_TABLE_t As Any
Type orxANIMSET_LINK_TABLE As __orxANIMSET_LINK_TABLE_t 


Extern "Windows"

/'* AnimSet module setup
 '/
Declare Sub orxAnimSet_Setup()

/'* Inits the AnimSet module
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxAnimSet_Init() As orxSTATUS 

/'* Exits from the AnimSet module
 '/
Declare Sub orxAnimSet_Exit()


/'* Creates an empty AnimSet
 * @param[in]		_u32Size														Storage size
 * return 			Created orxANIMSET / orxNULL
 '/
Declare Function orxAnimSet_Create(ByVal _u32Size As orxU32) As orxANIMSET Ptr

/'* Creates an animation set from config
 * @param[in]   _zConfigID                    Config ID
 * @return      orxANIMSET / orxNULL
 '/
Declare Function orxAnimSet_CreateFromConfig(ByVal _zConfigID As Const orxSTRING) As orxANIMSET Ptr

/'* Deletes an AnimSet
 * @param[in]		_pstAnimSet													AnimSet to delete
 '/
Declare Function orxAnimSet_Delete(ByVal _pstAnimSet As orxANIMSET Ptr) As orxSTATUS

/'* Adds a reference to an AnimSet
 * @param[in]		_pstAnimSet													Concerned AnimSet
 '/
Declare Sub orxAnimSet_AddReference(ByVal _pstAnimSet As orxANIMSET Ptr)

/'* Removes a reference from an AnimSet
 * @param[in]		_pstAnimSet													Concerned AnimSet
 '/
Declare Sub orxAnimSet_RemoveReference(ByVal _pstAnimSet As orxANIMSET Ptr)

/'* Clones an AnimSet Link Table
 * @param[in]		_pstAnimSet													AnimSet to clone
 * @return An internally allocated clone of the AnimSet
 '/
Declare Function orxAnimSet_CloneLinkTable(ByVal _pstAnimSet As Const orxANIMSET Ptr) As orxANIMSET_LINK_TABLE Ptr

/'* Deletes a Link table
 * @param[in]		_pstLinkTable											  Link table to delete (should have been created using the clone function)
 '/
Declare Sub orxAnimSet_DeleteLinkTable(ByVal _pstLinkTable As orxANIMSET_LINK_TABLE Ptr)


/'* Adds an Anim to an AnimSet
 * @param[in]		_pstAnimSet													Concerned AnimSet
 * @param[in]		_pstAnim														Anim to add
 * @return Anim handle in the specified AnimSet
 '/
Declare Function orxAnimSet_AddAnim(ByVal _pstAnimSet As orxANIMSET Ptr, _
                                    ByVal _pstAnim As orxANIM Ptr _
																	 ) As orxHandle

/'* Removes an Anim from an AnimSet
 * @param[in]		_pstAnimSet													Concerned AnimSet
 * @param[in]		_hAnimHandle												Handle of the anim to remove
 * @return 			orxSTATUS_SUCESS / orxSTATUS_FAILURE
 '/
Declare Function orxAnimSet_RemoveAnim(ByVal _pstAnimSet As orxANIMSET Ptr, _
                                       ByVal _hAnimHandle As orxHANDLE _
																			) As orxSTATUS

/'* Removes all Anim from the AnimSet
 * @param[in]		_pstAnimSet													Concerned AnimSet
 * @return 			orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxAnimSet_RemoveAllAnims(ByVal _pstAnimSet As orxANIMSET Ptr) As orxSTATUS


/'* Adds a link between two Anims of the AnimSet
 * @param[in]		_pstAnimSet													Concerned AnimSet
 * @param[in]		_hSrcAnim														Source Anim of the link
 * @param[in]		_hDstAnim														Destination Anim of the link
 * @return 			Handle of the created link / orxHANDLE_UNDEFINED
 '/
Declare Function orxAnimSet_AddLink(ByVal _pstAnimSet As orxANIMSET Ptr, _
                                    ByVal _hSrcAnim As orxHANDLE, _
																		ByVal _hDstAnim As orxHANDLE _
																	 ) As orxHANDLE

/'* Removes a link from the AnimSet
 * @param[in]		_pstAnimSet													Concerned AnimSet
 * @param[in]		_hLinkHandle												Handle of the link
 * @return			orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxAnimSet_RemoveLink(ByVal _pstAnimSet As orxANIMSET Ptr, _
                                       ByVal _hLinkHandle As orxHANDLE _
																			) As orxSTATUS

/'* Gets a direct link between two Anims, if exists
 * @param[in]		_pstAnimSet													Concerned AnimSet
 * @param[in]		_hSrcAnim														Handle of the source Anim
 * @param[in]		_hDstAnim														Handle of the destination Anim
 * @return			Handle of the direct link, orxHANDLE_UNDEFINED if none
 '/
Declare Function orxAnimSet_GetLink(ByVal _pstAnimSet As Const orxANIMSET Ptr, _
                                    ByVal _hSrcAnim As orxHANDLE, _
																		ByVal _hDstAnim As orxHANDLE _
																		) As orxHANDLE 

/'* Sets a link property
 * @param[in]		_pstAnimSet													Concerned AnimSet
 * @param[in]		_hLinkHandle												Handle of the concerned link
 * @param[in]		_u32Property												ID of the property to set
 * @param[in]		_u32Value                           Value of the property to set
 * @return			orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxAnimSet_SetLinkProperty(ByVal _pstAnimSet As orxANIMSET Ptr, _
                                            ByVal _hLinkHandle As orxHANDLE, _
																						ByVal _u32Property As orxU32 , _
																						ByVal _u32Value As orxU32 _
																					 ) As orxSTATUS

/'* Gets a link property
 * @param[in]		_pstAnimSet													Concerned AnimSet
 * @param[in]		_hLinkHandle 												Handle of the concerned link
 * @param[in]		_u32Property												ID of the property to get
 * @return			Property value / orx32_Undefined
 '/
Declare Function orxAnimSet_GetLinkProperty(ByVal _pstAnimSet As Const orxANIMSET Ptr, _
                                            ByVal _hLinkHandle As orxHANDLE , _
																						ByVal _u32Property As orxU32 _
																					 ) As orxU32

/'* Computes active Anim given current and destination Anim handles & a relative timestamp
 * @param[in]   _pstAnimSet                         Concerned AnimSet
 * @param[in]   _hSrcAnim                           Source (current) Anim handle
 * @param[in]   _hDstAnim                           Destination Anim handle, if none (auto mode) set it to orxHANDLE_UNDEFINED
 * @param[in,out] _pfTime                           Pointer to the current timestamp relative to the source Anim (time elapsed since the beginning of this anim)
 * @param[in,out] _pstLinkTable                     Anim Pointer link table (updated if AnimSet link table isn't static, when using loop counters for example)
 * @param[out] _pbCut                               Animation has been cut
 * @param[out] _pbClearTarget                       Animation has requested a target clearing
 * @return Current Anim handle. If it's not the source one, _pu32Time will contain the new timestamp, relative to the new Anim
'/
Declare Function orxAnimSet_ComputeAnim(ByVal _pstAnimSet As orxANIMSET Ptr, _
                                        ByVal _hSrcAnim As orxHANDLE, _ 
																				ByVal _hDstAnim As orxHANDLE, _
																				ByVal _pfTime As orxFLOAT Ptr, _
																				ByVal _pstLinkTable As orxANIMSET_LINK_TABLE Ptr, _
																				ByVal _pbCut As orxBOOL Ptr, _
																				ByVal _pbClearTarget As orxBOOL Ptr _
																			 ) As orxHANDLE

/'* AnimSet Anim get accessor
 * @param[in]		_pstAnimSet													Concerned AnimSet
 * @param[in]		_hAnimHandle												Anim handle
 * @return Anim pointer / orxNULL
 '/
Declare Function orxAnimSet_GetAnim(ByVal _pstAnimSet As Const orxANIMSET Ptr, _
                                    ByVal _hAnimHandle As orxHANDLE _
																	 ) As orxANIM Ptr

/'* AnimSet Anim storage size get accessor
 * @param[in]		_pstAnimSet													Concerned AnimSet
 * @return			AnimSet Storage size / orxU32_UNDEFINED
 '/
Declare Function orxAnimSet_GetAnimStorageSize(ByVal _pstAnimSet As Const orxANIMSET Ptr) As orxU32

/'* AnimSet Anim counter get accessor
 * @param[in]		_pstAnimSet													Concerned AnimSet
 * @return			Anim counter / orxU32_UNDEFINED
 '/
Declare Function orxAnimSet_GetAnimCounter(ByVal _pstAnimSet As Const orxANIMSET Ptr) As orxU32

/'* Gets animation handle from ID
 * @param[in]   _pstAnimSet                         Concerned AnimSet
 * @param[in]   _u32AnimID                          Animation ID (config's animation name CRC)
 * @return Anim handle / orxHANDLE_UNDEFINED
 '/
Declare Function orxAnimSet_GetAnimHandleFromID(ByVal _pstAnimSet As Const orxANIMSET Ptr, _
                                                ByVal _u32AnimID As orxU32 _
																							 ) As orxHANDLE

End Extern

#endif /' _orxANIMSET_BI '/

/'* @} '/
