/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxClock.h
 * @date 28/01/2004
 * @author iarwain@orx-project.org
 * 
 * @todo 
 * - Use double or int/fixed point for time to keep DT precision over time
 * - Add freezing behaviour
 '/

/'*
 * @addtogroup orxClock
 * 
 * Clock module
 * Module that handles clocks which is the low level kernel part of orx execution.
 * @{
 '/

#ifndef _orxCLOCK_BI
#define _orxCLOCK_BI


#include "orxInclude.bi"
#include "orx/core/orxSystem.bi"
#include "orx/utils/orxString.bi"


#define orxCLOCK_KU32_CLOCK_BANK_SIZE                 16          /'*< Clock bank size '/
#define orxCLOCK_KU32_TIMER_BANK_SIZE                 32          /'*< Timer bank size '/

#define orxCLOCK_KU32_FUNCTION_BANK_SIZE              16          /'*< Function bank size '/


/'* Clock type enum
 '/
Enum orxCLOCK_TYPE 
  orxCLOCK_TYPE_CORE = 0,
  orxCLOCK_TYPE_USER,

  orxCLOCK_TYPE_SECOND,

  orxCLOCK_TYPE_NUMBER,

  orxCLOCK_TYPE_NONE = orxENUM_NONE

End Enum



/'* Clock mod type enum
 '/
Enum orxCLOCK_MOD_TYPE 

  orxCLOCK_MOD_TYPE_FIXED = 0,                        /'*< The given DT will always be constant (= modifier value) '/
  orxCLOCK_MOD_TYPE_MULTIPLY,                         /'*< The given DT will be the real one * modifier '/
  orxCLOCK_MOD_TYPE_MAXED,                            /'*< The given DT will be the real one maxed by the modifier value '/

  orxCLOCK_MOD_TYPE_NUMBER,

  orxCLOCK_MOD_TYPE_NONE = orxENUM_NONE,
    
End Enum




/'* Clock priority
 '/
Enum orxCLOCK_PRIORITY
  orxCLOCK_PRIORITY_LOWEST = 0,
  orxCLOCK_PRIORITY_LOWER,
  orxCLOCK_PRIORITY_LOW,
  orxCLOCK_PRIORITY_NORMAL,
  orxCLOCK_PRIORITY_HIGH,
  orxCLOCK_PRIORITY_HIGHER,
  orxCLOCK_PRIORITY_HIGHEST,

  orxCLOCK_PRIORITY_NUMBER,

  orxCLOCK_PRIORITY_NONE = orxENUM_NONE,

End Enum

/'* Clock info structure
 '/
Type __orxCLOCK_INFO_t
  As orxCLOCK_TYPE     eType                            /'*< Clock type : 4 '/
  As orxFLOAT          fTickSize                        /'*< Clock tick size (in seconds) : 8 '/
  As orxCLOCK_MOD_TYPE eModType                         /'*< Clock mod type : 12 '/
  As orxFLOAT          fModValue                        /'*< Clock mod value : 16 '/
  As orxFLOAT          fDT                              /'*< Clock DT (time ellapsed between 2 clock calls in seconds) : 20 '/
  As orxFLOAT          fTime                            /'*< Clock time : 24 '/ 
End Type

Type orxCLOCK_INFO As __orxCLOCK_INFO_t


/'* Event enum
 '/
Enum orxCLOCK_EVENT
  orxCLOCK_EVENT_RESTART = 0,                         /'*< Event sent when a clock restarts '/
  orxCLOCK_EVENT_RESYNC,                              /'*< Event sent when a clock recyncs '/
  orxCLOCK_EVENT_PAUSE,                               /'*< Event sent when a clock is paused '/
  orxCLOCK_EVENT_UNPAUSE,                             /'*< Event sent when a clock is unpaused '/

  orxCLOCK_EVENT_NUMBER,

  orxCLOCK_EVENT_NONE = orxENUM_NONE

End Enum


/'* Clock structure '/
Type __orxCLOCK_t As Any
Type orxCLOCK As __orxCLOCK_t

/'* Clock callback function type to use with clock bindings '/
Type orxCLOCK_FUNCTION As Sub(ByVal _pstClockInfo As Const orxCLOCK_INFO Ptr, ByVal _pContext As Any Ptr)


Extern "Windows"

/'* Clock module setup
 '/
Declare Sub orxClock_Setup()

/'* Inits the clock module
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxClock_Init() As orxSTATUS

/'* Exits from the clock module
 '/
Declare Sub orxClock_Exit()


/'* Updates the clock system
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxClock_Update() As orxSTATUS 

/'* Creates a clock
 * @param[in]   _fTickSize                            Tick size for the clock (in seconds)
 * @param[in]   _eType                                Type of the clock
 * @return      orxCLOCK / orxNULL
 '/
Declare Function orxClock_Create(ByVal _fTickSize As orxFLOAT, _
                                 ByVal _eType As orxCLOCK_TYPE _
                                ) As orxCLOCK Ptr

/'* Creates a clock from config
 * @param[in]   _zConfigID    Config ID
 * @ return orxCLOCK / orxNULL
 '/
Declare Function orxClock_CreateFromConfig(ByVal _zConfigID As Const orxSTRING) As orxCLOCK Ptr

/'* Deletes a clock
 * @param[in]   _pstClock                             Concerned clock
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxClock_Delete(ByVal _pstClock As orxCLOCK Ptr) As  orxSTATUS

/'* Resyncs a clock (accumulated DT => 0)
 * @param[in]   _pstClock                             Concerned clock
 '/
Declare Function orxClock_Resync(ByVal _pstClock As orxCLOCK Ptr)  As orxSTATUS

/'* Resyncs all clocks (accumulated DT => 0)
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxClock_ResyncAll() As orxSTATUS

/'* Restarts a clock
 * @param[in]   _pstClock                             Concerned clock
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxClock_Restart(ByVal _pstClock As orxCLOCK Ptr)As orxSTATUS

/'* Pauses a clock
 * @param[in]   _pstClock                             Concerned clock
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxClock_Pause(ByVal _pstClock As orxCLOCK Ptr)As orxSTATUS

/'* Unpauses a clock
 * @param[in]   _pstClock                             Concerned clock
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxClock_Unpause(ByVal _pstClock As orxCLOCK Ptr) As orxSTATUS

/'* Is a clock paused?
 * @param[in]   _pstClock                             Concerned clock
 * @return      orxTRUE if paused, orxFALSE otherwise
 '/
Declare Function orxClock_IsPaused(ByVal _pstClock As Const orxCLOCK Ptr)As orxBOOL

/'* Gets clock info
 * @param[in]   _pstClock                             Concerned clock
 * @return      orxCLOCK_INFO / orxNULL
 '/
Declare Function orxClock_GetInfo(ByVal _pstClock As Const orxCLOCK Ptr) As Const orxCLOCK_INFO Ptr

/'* Gets clock from its info
 * @param[in]   _pstClockInfo                         Concerned clock info
 * @return      orxCLOCK / orxNULL
 '/
Declare Function orxClock_GetFromInfo(ByVal _pstClockInfo As Const orxCLOCK_INFO Ptr) As orxCLOCK Ptr


/'* Sets a clock modifier
 * @param[in]   _pstClock                             Concerned clock
 * @param[in]   _eModType                             Modifier type
 * @param[in]   _fModValue                            Modifier value
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxClock_SetModifier(ByVal _pstClock As orxCLOCK Ptr, _
                                      ByVal _eModType As orxCLOCK_MOD_TYPE, _
																			ByVal _fModValue As orxFLOAT _
																		  ) As orxSTATUS

/'* Sets a clock tick size
 * @param[in]   _pstClock                             Concerned clock
 * @param[in]   _fTickSize                            Tick size
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxClock_SetTickSize(ByVal _pstClock As orxCLOCK Ptr, _
                                      ByVal _fTickSize As orxFLOAT _
																			) As orxSTATUS


/'* Registers a callback function to a clock
 * @param[in]   _pstClock                             Concerned clock
 * @param[in]   _pfnCallback                          Callback to register
 * @param[in]   _pContext                             Context that will be transmitted to the callback when called
 * @param[in]   _eModuleID                            ID of the module related to this callback
 * @param[in]   _ePriority                            Priority for the function
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxClock_Register(ByVal _pstClock As orxCLOCK Ptr, _
                                   ByVal _pfnCallback As Const orxCLOCK_FUNCTION, _
																	 ByVal _pContext As Any Ptr, _
																	 ByVal _eModuleID As orxMODULE_ID, _
																	 ByVal _ePriority As orxCLOCK_PRIORITY _
																	) As orxSTATUS

/'* Unregisters a callback function from a clock
 * @param[in]   _pstClock                             Concerned clock
 * @param[in]   _pfnCallback                          Callback to remove
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxClock_Unregister(ByVal _pstClock As orxCLOCK Ptr, _
                                     ByVal _pfnCallback As Const orxCLOCK_FUNCTION _
																		) As orxSTATUS

/'* Gets a callback function context
 * @param[in]   _pstClock                             Concerned clock
 * @param[in]   _pfnCallback                          Concerned callback
 * @return      Registered context
 '/
Declare Function orxClock_GetContext(ByVal _pstClock As Const orxCLOCK Ptr, _
                                                           ByVal _pfnCallback As Const orxCLOCK_FUNCTION) As Any Ptr

/'* Sets a callback function context
 * @param[in]   _pstClock                             Concerned clock
 * @param[in]   _pfnCallback                          Concerned callback
 * @param[in]   _pContext                             Context that will be transmitted to the callback when called
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxClock_SetContext(ByVal _pstClock As orxCLOCK Ptr, _
                                                           ByVal _pfnCallback As Const orxCLOCK_FUNCTION, _
																													 ByVal _pContext As Any Ptr _
																													 ) As orxSTATUS


/'* Finds a clock given its tick size and its type
 * @param[in]   _fTickSize                            Tick size of the desired clock (in seconds)
 * @param[in]   _eType                                Type of the desired clock
 * @return      orxCLOCK / orxNULL
 '/
Declare Function orxClock_FindFirst(ByVal _fTickSize As orxFLOAT, _
                                                          ByVal _eType As orxCLOCK_TYPE _
																												 ) As orxCLOCK Ptr

/'* Finds next clock of same type/tick size
 * @param[in]   _pstClock                             Concerned clock
 * @return      orxCLOCK / orxNULL
 '/
Declare Function orxClock_FindNext(ByVal _pstClock As Const orxCLOCK Ptr) As orxCLOCK Ptr

/'* Gets next existing clock in list (can be used to parse all existing clocks)
 * @param[in]   _pstClock                             Concerned clock
 * @return      orxCLOCK / orxNULL
 '/
Declare Function orxClock_GetNext(ByVal _pstClock As Const orxCLOCK Ptr) As orxCLOCK Ptr

/'* Gets clock given its name
 * @param[in]   _zName          Clock name
 * @return      orxCLOCK / orxNULL
 '/
Declare Function orxClock_Get(ByVal _zName As Const orxSTRING) As orxCLOCK Ptr


/'* Adds a timer function to a clock
 * @param[in]   _pstClock                             Concerned clock
 * @param[in]   _pfnCallback                          Concerned timer callback
 * @param[in]   _fDelay                               Timer's delay between 2 calls, must be strictly positive
 * @param[in]   _s32Repetition                        Number of times this timer should be called before removed, -1 for infinite
 * @param[in]   _pContext                             Context that will be transmitted to the callback when called
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxClock_AddTimer(ByVal _pstClock As orxCLOCK Ptr, _
                                                         ByVal _pfnCallback As Const orxCLOCK_FUNCTION, _
																												 ByVal _fDelay As orxFLOAT , _
																												 ByVal _s32Repetition As orxS32, _
																												 ByVal _pContext As Any Ptr _
																												) As orxSTATUS

/'* Removes a timer function from a clock
 * @param[in]   _pstClock                             Concerned clock
 * @param[in]   _pfnCallback                          Concerned timer callback to remove
 * @param[in]   _fDelay                               Delay between 2 calls of the timer to remove, -1.0f for removing all occurences regardless of their respective delay
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxClock_RemoveTimer(ByVal _pstClock As orxCLOCK Ptr, _
                                                            ByVal _pfnCallback As Const orxCLOCK_FUNCTION, _
																														ByVal _fDelay As orxFLOAT _
																													 ) As orxSTATUS

/'* Adds a global timer function (ie. using the main core clock)
 * @param[in]   _pfnCallback                          Concerned timer callback
 * @param[in]   _fDelay                               Timer's delay between 2 calls, must be strictly positive
 * @param[in]   _s32Repetition                        Number of times this timer should be called before removed, -1 for infinite
 * @param[in]   _pContext                             Context that will be transmitted to the callback when called
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxClock_AddGlobalTimer(ByVal _pfnCallback As Const orxCLOCK_FUNCTION, _
                                         ByVal _fDelay As orxFLOAT, _
																				 ByVal _s32Repetition As orxS32, _
																				 ByVal _pContext As Any Ptr _
																				 ) As orxSTATUS

/'* Removes a global timer function (ie. from the main core clock)
 * @param[in]   _pfnCallback                          Concerned timer callback to remove
 * @param[in]   _fDelay                               Delay between 2 calls of the timer to remove, -1.0f for removing all occurences regardless of their respective delay
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxClock_RemoveGlobalTimer(ByVal _pfnCallback As Const orxCLOCK_FUNCTION, _
                                            ByVal _fDelay As orxFLOAT _
																						) As orxSTATUS

End Extern

#endif /' _orxCLOCK_BI '/

/'* @} '/
