/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxMath.h
 * @date 30/03/2005
 * @author iarwain@orx-project.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxMath
 * 
 * Math module
 * Contains all base math functions
 *
 * @{
 '/

#ifndef _orxMATH_BI
#define _orxMATH_BI


#include "orxInclude.bi"
/'* Maths related includes
 '/
#include "crt/math.bi"
#include "crt/stdlib.bi"


/'* Public macro
 '/

/'* Lerps between two values given a parameter T [0, 1]
 * @param[in]   A                               First value (will be selected for T = 0)
 * @param[in]   B                               Second value (will be selected for T = 1)
 * @param[in]   T                               Lerp coefficient parameter [0, 1]
 * @return      Lerped value
 '/
#define orxLERP(A, B, T)          ((A) + ((T) * ((B) - (A))))


/'* Gets minimum between two values
 * @param[in]   A                               First value
 * @param[in]   B                               Second value
 * @return      Minimum between A & B
 '/
#define orxMIN(A, B)              (Iif((A) > (B) ),(B),(A))

/'* Gets maximum between two values
 * @param[in]   A                               First value
 * @param[in]   B                               Second value
 * @return      Maximum between A & B
 '/
#define orxMAX(A, B)              (Iif((A) > (B) ),(B),(A))

/'* Gets clamped value between two boundaries
 * @param[in]   V                               Value to clamp
 * @param[in]   MIN                             Minimum boundary
 * @param[in]   MAX                             Maximum boundary
 * @return      Clamped value between MIN & MAX
 '/
#define orxCLAMP(V, MIN, MAX)     orxMAX(orxMIN(V, MAX), MIN)

/'* Gets circularly clamped (for ring spaces) value between two boundaries [MIN, MAX[
 * @param[in]   V                               Value to clamp
 * @param[in]   MIN                             Minimum boundary
 * @param[in]   MAX                             Maximum boundary
 * @return      Circularly clamped value between MIN & MAX
 '/
#define orxCIRCULAR_CLAMP_INC_MIN(V, MIN, MAX)   _
Do 																	             :_
	While((V) < (MIN))					                   :_
		(V) += ((MAX) - (MIN))	                     :_
	Wend 													                 :_
	While((V) >= (MAX)) 			                     :_
		 (V) -= ((MAX) - (MIN)) 	                   :_
	Wend 													                 :_
Loop While(0)

/'* Gets circularly clamped (for ring spaces) value between two boundaries ]MIN, MAX]
 * @param[in]   V                               Value to clamp
 * @param[in]   MIN                             Minimum boundary
 * @param[in]   MAX                             Maximum boundary
 * @return      Circularly clamped value between MIN & MAX
 '/
#define orxCIRCULAR_CLAMP_INC_MAX(V, MIN, MAX) _
Do																	:_
	While((V) <= (MIN))				        :_
		(V) += ((MAX) - (MIN))	        :_
	Wend														  :_
	While((V) > (MAX))				        :_
		(V) -= ((MAX) - (MIN))		      :_
	Wend														  :_
Loop While(0)

/'* Converts an orxFLOAT to an orxU32
 * @param[in]   V                               Value to convert
 * @return      Converted value
 '/
#define orxF2U(V)                 (Cast(orxU32,(V)))

/'* Converts an orxFLOAT to an orxS32
 * @param[in]   V                               Value to convert
 * @return      Converted value
 '/
#define orxF2S(V)                 (Cast(orxS32,(V)))

/'* Converts an orxU32 to an orxFLOAT
 * @param[in]   V                               Value to convert
 * @return      Converted value
 '/
#define orxU2F(V)                 (Cast(orxFLOAT,(V)))

/'* Converts an orxS32 to an orxFLOAT
 * @param[in]   V                               Value to convert
 * @return      Converted value
 '/
#define orxS2F(V)                 (Cast(orxFLOAT,(V)))


/'** Module functions *** '/

Extern "Windows"

/'* Inits the random seed with an orxFLOAT
 * @param[in]   _u32Seed                        Value to use as seed for random number generation
 '/

Declare Sub orxMath_InitRandom(ByVal _u32Seed As orxU32)

/'* Gets a random orxFLOAT value
 * @param[in]   _fMin                           Minimum boundary
 * @param[in]   _fMax                           Maximum boundary
 * @return      Random value
 '/
Declare Function orxMath_GetRandomFloat(ByVal _fMin As orxFLOAT,ByVal _fMax As orxFLOAT) As orxFLOAT

/'* Gets a random orxU32 value
 * @param[in]   _u32Min                         Minimum boundary
 * @param[in]   _u32Max                         Maximum boundary
 * @return      Random value
 '/
Declare Function orxMath_GetRandomU32(ByVal _u32Min As orxU32,ByVal _u32Max As orxU32) As orxU32

/'* Gets a random orxS32 value
 * @param[in]   _s32Min                         Minimum boundary
 * @param[in]   _s32Max                         Maximum boundary
 * @return      Random value
 '/
Declare Function orxMath_GetRandomS32(ByVal _s32Min As orxS32,ByVal _s32Max As orxS32) As orxS32

End Extern

/'** Inlined functions *** '/

/'* Gets the counts of bit in an orxU32
 * @param[in]   _u32Value                       Value to process
 * @return      Number of bits that are set in the value
 '/
#macro orxMath_GetBitCount(_u32Value,_result)

  _u32Value -= ((_u32Value Shr 1) And &H55555555)
  _u32Value = (((_u32Value Shr 2) And &H33333333) + (_u32Value And &H33333333))
  _u32Value = (((_u32Value Shr 4) + _u32Value) And &H0f0f0f0f)
  _u32Value += (_u32Value Shr 8)
  _u32Value += (_u32Value Shr 16)
  _result = (_u32Value And &H0000003f)

#endmacro

/'** Math Definitions **'/

#define orxMATH_KF_SQRT_2         				orx2F(1.414213562f)           /'*< Sqrt(2) constant '/
#define orxMATH_KF_EPSILON        			orx2F(0.0001f)                /'*< Epsilon constant '/
#define orxMATH_KF_2_PI           					orx2F(6.283185307f)           /'*< 2 PI constant '/
#define orxMATH_KF_PI             						orx2F(3.141592654f)           /'*< PI constant '/
#define orxMATH_KF_PI_BY_2        				orx2F(1.570796327f)           /'*< PI / 2 constant '/
#define orxMATH_KF_PI_BY_4        				orx2F(0.785398163f)           /'*< PI / 4 constant '/
#define orxMATH_KF_DEG_TO_RAD     	orx2F(3.141592654f / 180.0f)  /'*< Degree to radian conversion constant '/
#define orxMATH_KF_RAD_TO_DEG     	orx2F(180.0f / 3.141592654f)  /'*< Radian to degree conversion constant '/


/'** Trigonometric function **'/

/'* Gets a cosine
 * @param[in]   _fOp                            Input radian angle value
 * @return      Cosine of the given angle
 '/
#macro orxMath_Cos(_fOp,_result)
 
#ifdef __orxMSVC__
  
  /' Updates result '/
  _result = cosf(_fOp)

#else /' __orxMSVC__ '/

  /' Updates result '/
  _result = cosf(_fOp)

#endif /' __orxMSVC__ '/

  /' Done! '/
  
#endmacro


/'* Gets a sine
 * @param[in]   _fOp                            Input radian angle value
 * @return      Sine of the given angle
 '/
#macro orxMath_Sin(_fOp,_result)

#ifdef __orxMSVC__
  
  /' Updates result '/
  _result = sinf(_fOp)

#else /' __orxMSVC__ '/

  /' Updates result '/
  _result = sinf(_fOp)

#endif /' __orxMSVC__ '/

  /' Done! '/
 #endmacro 


/'* Gets a tangent
 * @param[in]   _fOp                            Input radian angle value
 * @return      Tangent of the given angle
 '/
#macro orxMath_Tan(_fOp,_result)  

#ifdef __orxMSVC__
  
  /' Updates result '/
  _result = tanf(_fOp)

#else /' __orxMSVC__ '/

  /' Updates result '/
  _result = tanf(_fOp)

#endif /' __orxMSVC__ '/

  /' Done! '/
  
#endmacro


/'* Gets an arccosine
 * @param[in]   _fOp                            Input radian angle value
 * @return      Arccosine of the given angle
 '/
#macro orxMath_ACos(_fOp,_result)

#ifdef __orxMSVC__
  
  /' Updates result '/
  _result = acosf(_fOp)

#else /' __orxMSVC__ '/

  /' Updates result '/
  _result = acosf(_fOp)

#endif /' __orxMSVC__ '/

  /' Done! '/
  

#endmacro


/'* Gets an arcsine
 * @param[in]   _fOp                            Input radian angle value
 * @return      Arcsine of the given angle
 '/
#macro orxMath_ASin(_fOp,_result)

#ifdef __orxMSVC__
  
  /' Updates result '/
  _result = asinf(_fOp)

#else /' __orxMSVC__ '/

  /' Updates result '/
  _result = asinf(_fOp)

#endif /' __orxMSVC__ '/

  /' Done! '/
  

#endmacro


/'* Gets an arctangent
 * @param[in]   _fOp1                           First operand
 * @param[in]   _fOp2                           Second operand
 * @return      Arctangent of the given angle
 '/
#macro orxMath_ATan(_fOp1,_fOp2,_result)

#ifdef __orxMSVC__
  
  /' Updates result '/
  _result = atan2f(_fOp1, _fOp2)

#else /' __orxMSVC__ '/

  /' Updates result '/
  _result = atan2f(_fOp1, _fOp2)

#endif /' __orxMSVC__ '/

  /' Done! '/
  
#endmacro



/'** Misc functions **'/

/'* Gets a square root
 * @param[in]   _fOp                            Input value
 * @return      Square root of the given value
 '/
#macro orxMath_Sqrt(_fOp,_result)
 
 
 /' Updates result '/
  _result = sqrtf(_fOp)

  /' Done! '/
  
#endmacro


/'* Gets a floored value
 * @param[in]   _fOp                            Input value
 * @return      Floored value
 '/
#macro orxMath_Floor(_fOp,_result)  

  /' Updates result '/
  _result = floorf(_fOp)

  /' Done! '/
  
#endmacro


/'* Gets a ceiled value
 * @param[in]   _fOp                            Input value
 * @return      Ceiled value
 '/
#macro orxMath_Ceil(_fOp,_result)

  /' Updates result '/
  _result = ceilf(_fOp)

  /' Done! '/
  
#endmacro


/'* Gets a rounded value
 * @param[in]   _fOp                            Input value
 * @return      Rounded value
 '/
#macro orxMath_Round(_fOp,_result)  

#ifdef __orxMSVC__
  
  /' Updates result '/
  _result = Iif(fmodf(_fOp, orxFLOAT_1) >= orx2F(0.5f),ceilf(_fOp),floorf(_fOp))

#else /' __orxMSVC__ '/

  /' Updates result '/
  _result = rintf(_fOp)

#endif /' __orxMSVC__ '/

  /' Done! '/  

#endmacro


/'* Gets a modulo value
 * @param[in]   _fOp1                           Input value
 * @param[in]   _fOp2                           Modulo value
 * @return      Modulo value
 '/
#macro orxMath_Mod(_fOp1,_fOp2,_result)  

  /' Updates result '/
  _result = fmodf(_fOp1, _fOp2)

  /' Done! '/
  
#endmacro

/'* Gets a powed value
 * @param[in]   _fOp                            Input value
 * @param[in]   _fExp                           Exponent value
 * @return      Powed value
 '/
#macro orxMath_Pow(_fOp,_fExp,_result)		

  /' Updates result '/
  _result = powf(_fOp, _fExp)

  /' Done! '/
  
#endmacro


/'* Gets an absolute value
 * @param[in]   _fOp   Input value
 * @return      Absolute value
 '/
#macro orxMath_Abs(_fOp,_result)

  /' Done! '/
  _result = fabsf(_fOp)

#endmacro

#endif /' _orxMATH_BI '/

/'* @} '/
