/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxOBox.h
 * @date 03/10/2008
 * @author iarwain@orx-project.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxOBox
 * 
 * Oriented box module
 * Module that handles oriented boxes
 *
 * @{
 '/


#ifndef _orxOBOX_BI
#define _orxOBOX_BI

#include "orxInclude.bi"
#include "orx/math/orxVector.bi"


/'* Public oriented box structure
'/
Type __orxOBOX_t

    As orxVECTOR vPosition /'*< Position vector  : 12 '/
    As orxVECTOR vPivot    /'*< Pivot vector     : 24 '/
    As orxVECTOR vX        /'*< X axis vector    : 36 '/
    As orxVECTOR vY        /'*< Y axis vector    : 48 '/
    As orxVECTOR vZ        /'*< Z axis vector    : 60 '/

End Type

Type orxOBOX As __orxOBOX_t


/' *** OBox inlined functions *** '/


/'* Sets 2D oriented box values
 * @param[out]  _pstRes                       OBox to set
 * @param[in]   _pvWorldPosition              World space position vector
 * @param[in]   _pvPivot                      Pivot vector
 * @param[in]   _pvSize                       Size vector
 * @param[in]   _fAngle                       Z-axis angle (radians)
 * @return      orxOBOX / orxNULL
 '/
#macro orxOBox_2DSet(_pstRes,_pvWorldPosition,_pvPivot,_pvSize,_fAngle,_result)

Scope
		Dim As orxFLOAT fCos
		Dim As orxFLOAT fSin
		Dim As orxVECTOR Ptr vTEMP

		/' Checks '/
		orxASSERT(_pstRes <> orxNULL)
		orxASSERT(_pvWorldPosition <> orxNULL)
		orxASSERT(_pvPivot <> orxNULL)

		/' Gets cosine and sine '/
		orxMath_Cos(_fAngle,fCos)
		orxMath_Sin(_fAngle,fSin)

		/' Sets axis '/
		orxVector_Set(@(_pstRes->vX), fCos * _pvSize->fX, fSin * _pvSize->fX, orxFLOAT_0,vTEMP)
		orxVector_Set(@(_pstRes->vY), -fSin * _pvSize->fY, fCos * _pvSize->fY, orxFLOAT_0,vTEMP)
		orxVector_Set(@(_pstRes->vZ), orxFLOAT_0, orxFLOAT_0, _pvSize->fZ,vTEMP)

		/' Sets pivot '/
		orxVector_Set(@(_pstRes->vPivot), (fCos * _pvPivot->fX) - (fSin * _pvPivot->fY), (fSin * _pvPivot->fX) + (fCos * _pvPivot->fY), _pvPivot->fZ,vTEMP)

		/' Sets box position '/
		orxVector_Copy(@(_pstRes->vPosition), _pvWorldPosition,vTEMP)

		/' Done! '/
		_result = _pstRes
End Scope
#endmacro

/'* Copies an OBox onto another one
 * @param[out]  _pstDst                       OBox to copy to (destination)
 * @param[in]   _pstSrc                       OBox to copy from (destination)
 * @return      Destination OBox
 '/
#macro orxOBox_Copy(_pstDst,_pstSrc,_result)

  /' Checks '/
  orxASSERT(_pstDst <> orxNULL)
  orxASSERT(_pstSrc <> orxNULL)

  /' Copies it '/
  orxMemory_Copy(_pstDst, _pstSrc, SizeOf(orxOBOX))

  /' Done! '/
  _result = _pstDst

#endmacro

/'* Gets OBox center position
 * @param[in]   _pstOp                        Concerned OBox
 * @param[out]  _pvRes                        Center position
 * @return      Center position vector
 '/
#macro orxOBox_GetCenter(_pstOp,_pvRes,_result)

  /' Checks '/
  orxASSERT(_pstOp <> orxNULL)
  orxASSERT(_pvRes <> orxNULL)
  Dim As orxVECTOR Ptr vTEMP
	Dim As orxVECTOR Ptr vTEMP1
	
	orxVector_Add(_pvRes, @(_pstOp->vX), @(_pstOp->vY),vTEMP1)	
  /' Gets box center '/
  orxVector_Add(_pvRes, vTEMP1,@(_pstOp->vZ),vTEMP)
  orxVector_Mulf(_pvRes, _pvRes, orx2F(0.5f),vTEMP)
	
	orxVector_Add(_pvRes, _pvRes, @(_pstOp->vPosition),vTEMP1)
  orxVector_Sub(_pvRes, vTEMP1, @(_pstOp->vPivot),vTEMP)

  /' Done! '/
  _result = _pvRes

#endmacro

/'* Moves an OBox
 * @param[out]  _pstRes                       OBox where to store result
 * @param[in]   _pstOp                        OBox to move
 * @param[in]   _pvMove                       Move vector
 * @return      Moved OBox
 '/
#macro orxOBox_Move(_pstRes,_pstOp,_pvMove,_result)

  /' Checks '/
  orxASSERT(_pstRes <> orxNULL)
  orxASSERT(_pstOp <> orxNULL)
  orxASSERT(_pvMove <> orxNULL)
  Dim As orxVECTOR Ptr vTEMP
  /' Updates result '/
  orxVector_Add(@(_pstRes->vPosition), @(_pstOp->vPosition), _pvMove,vTEMP)

  /' Done! '/
  _result = _pstRes

#endmacro

/'* Rotates in 2D an OBox
 * @param[out]  _pstRes                       OBox where to store result
 * @param[in]   _pstOp                        OBox to rotate (its Z-axis vector will be unchanged)
 * @param[in]   _fAngle                       Z-axis rotation angle (radians)
 * @return      Rotated OBox
 '/
#macro orxOBox_2DRotate(_pstRes,_pstOp,_fAngle,_result)

Scope
		Dim As orxFLOAT fSin
		Dim As orxFLOAT fCos
    Dim As orxVECTOR Ptr vTEMP
		/' Checks '/
		orxASSERT(_pstRes <> orxNULL)
		orxASSERT(_pstOp <> orxNULL)
    
		/' Gets cos & sin of angle '/
		fCos = orxMath_Cos(_fAngle)
		fSin = orxMath_Sin(_fAngle)

		/' Updates axis '/
		orxVector_Set(@(_pstRes->vX), (fCos * _pstRes->vX.fX) - (fSin * _pstRes->vX.fY), (fSin * _pstRes->vX.fX) + (fCos * _pstRes->vX.fY), _pstRes->vX.fZ,vTEMP)
		orxVector_Set(@(_pstRes->vY), (fCos * _pstRes->vY.fX) - (fSin * _pstRes->vY.fY), (fSin * _pstRes->vY.fX) + (fCos * _pstRes->vY.fY), _pstRes->vY.fZ,vTEMP)

		/' Updates pivot '/
		orxVector_Set(@(_pstRes->vPivot), (fCos * _pstRes->vPivot.fX) - (fSin * _pstRes->vPivot.fY), (fSin * _pstRes->vPivot.fX) + (fCos * _pstRes->vPivot.fY), _pstRes->vPivot.fZ,vTEMP)

		/' Done! '/
		_result = _pstRes;	
End Scope

#endmacro

/'* Is position inside oriented box test
 * @param[in]   _pstBox                       Box to test against position
 * @param[in]   _pvPosition                   Position to test against the box
 * @return      orxTRUE if position is inside the box, orxFALSE otherwise
 '/
#macro orxOBox_IsInside(_pstBox,_pvPosition,_result)

Scope
		_result = orxFALSE
		Dim As orxFLOAT  fProj
		Dim As orxVECTOR  vToPos
    Dim As orxVECTOR Ptr vTEMP
		Dim As orxVECTOR Ptr vTEMP1
		Dim As orxFLOAT fTEMP
		
		/' Checks '/
		orxASSERT(_pstBox <> orxNULL)
		orxASSERT(_pvPosition <> orxNULL)

		/' Gets origin to position vector '/
		orxVector_Sub(@vToPos, @(_pstBox->vPosition),@(_pstBox->vPivot),vTEMP1)
		orxVector_Sub(@vToPos, _pvPosition, vTEMP1, vTEMP)

		/' Z-axis test '/
		orxVector_Dot(@vToPos, @(_pstBox->vZ),fProj)
		orxVector_GetSquareSize(@(_pstBox->vZ),fTEMP)
		If( (fProj  >= orxFLOAT_0) AndAlso _
		 (fProj <= fTEMP)) Then
		
			/' X-axis test '/
			orxVector_Dot(@vToPos, @(_pstBox->vX),fProj)
			orxVector_GetSquareSize(@(_pstBox->vX),fTEMP)
			If((fProj >= orxFLOAT_0) AndAlso (fProj <= fTEMP)) Then
			
				/' Y-axis test '/
				orxVector_Dot(@vToPos, @(_pstBox->vY),fProj)
				orxVector_GetSquareSize(@(_pstBox->vY),fTEMP)
				If ((fProj >= orxFLOAT_0) AndAlso (fProj <= fTEMP)) Then
				
					/' Updates result '/
					_result = orxTRUE
				End If
			End If
		End If

		/' Done! '/
End Scope
#endmacro

/'* Is 2D position inside oriented box test
 * @param[in]   _pstBox                       Box to test against position
 * @param[in]   _pvPosition                   Position to test against the box (no Z-test)
 * @return      orxTRUE if position is inside the box, orxFALSE otherwise
 '/
#macro orxOBox_2DIsInside(_pstBox,_pvPosition,_result)

Scope 
		_result = orxFALSE
		Dim As orxFLOAT            fProj
		Dim As orxVECTOR           vToPos
    Dim As orxVECTOR Ptr vTEMP
		Dim As orxVECTOR Ptr vTEMP1
		Dim As orxFLOAT fTEMP
		
		/' Checks '/
		orxASSERT(_pstBox <> orxNULL)
		orxASSERT(_pvPosition <> orxNULL)

		/' Gets origin to position vector '/
		orxVector_Sub(@vToPos, @(_pstBox->vPosition), @(_pstBox->vPivot),vTEMP1)
		orxVector_Sub(@vToPos, _pvPosition, vTEMP1,vTEMP)

		/' X-axis test '/
		orxVector_Dot(@vToPos, @(_pstBox->vX),fProj)
		orxVector_GetSquareSize(@(_pstBox->vX),fTEMP)
		If ((fProj >= orxFLOAT_0) AndAlso (fProj <= fTEMP)) Then
		
			/' Y-axis test '/
			orxVector_Dot(@vToPos, @(_pstBox->vY),fProj)
			orxVector_GetSquareSize(@(_pstBox->vY),fTEMP)
			If((fProj  >= orxFLOAT_0) AndAlso (fProj <= fTEMP)) Then    
				/' Updates result '/
				_result = orxTRUE
			End If
			
		End If

  /' Done! '/
End Scope  

#endmacro


/'* Tests oriented 2D box intersection (simple Z-axis test, to use with Z-axis aligned orxOBOX)
 * @param[in]   _pstBox1                      First box operand
 * @param[in]   _pstBox2                      Second box operand
 * @return      orxTRUE if boxes intersect, orxFALSE otherwise
 '/
#macro orxOBox_2DTestIntersection(_pstBox1,_pstBox2,_result)

Scope
		/' Checks '/
		orxASSERT(_pstBox1 <> orxNULL)
		orxASSERT(_pstBox2 <> orxNULL)
		orxASSERT((_pstBox1->vZ.fX = orxFLOAT_0) AndAlso (_pstBox1->vZ.fX = orxFLOAT_0))
		orxASSERT((_pstBox1->vZ.fY = orxFLOAT_0) AndAlso (_pstBox1->vZ.fY = orxFLOAT_0))
		orxASSERT((_pstBox1->vZ.fZ >= orxFLOAT_0) AndAlso (_pstBox1->vZ.fZ >= orxFLOAT_0))

		/' Z intersected? '/
		If((_pstBox2->vPosition.fZ + _pstBox2->vZ.fZ >= _pstBox1->vPosition.fZ) AndAlso _
		(_pstBox2->vPosition.fZ <= _pstBox1->vPosition.fZ + _pstBox1->vZ.fZ)) Then
		
			DIm As orxU32            i
			Dim As orxVECTOR         vOrigin1
			Dim As orxVECTOR         vOrigin2
			Dim As orxVECTOR         Ptr pvOrigin1 = @vOrigin1
			Dim As orxVECTOR         Ptr pvOrigin2 = @vOrigin2
			Dim As orxVECTOR         Ptr pvTemp
			
			Const orxOBOX Ptr pstBox1 = _pstBox1
			Const orxOBOX Ptr pstBox2 = _pstBox2
			Const orxOBOX Ptr pstTemp

			/' Computes boxes origins '/
			vOrigin1.fX = _pstBox1->vPosition.fX - pstBox1->vPivot.fX
			vOrigin1.fY = _pstBox1->vPosition.fY - pstBox1->vPivot.fY
			vOrigin2.fX = _pstBox2->vPosition.fX - pstBox2->vPivot.fX
			vOrigin2.fY = _pstBox2->vPosition.fY - pstBox2->vPivot.fY

			/' Test each box against the other '/
			i = 2
			_result = orxTRUE
			While (i <> 0)
				Dim As orxVECTOR           vToCorner(0 To 3)
				Const pvAxis As orxVECTOR Ptr
				Dim As orxU32              j

				/' Gets to-corner vectors '/
				vToCorner[0].fX = pvOrigin2->fX - pvOrigin1->fX
				vToCorner[0].fY = pvOrigin2->fY - pvOrigin1->fY
				vToCorner[1].fX = vToCorner[0].fX + pstBox2->vX.fX
				vToCorner[1].fY = vToCorner[0].fY + pstBox2->vX.fY
				vToCorner[2].fX = vToCorner[1].fX + pstBox2->vY.fX
				vToCorner[2].fY = vToCorner[1].fY + pstBox2->vY.fY
				vToCorner[3].fX = vToCorner[0].fX + pstBox2->vY.fX
				vToCorner[3].fY = vToCorner[0].fY + pstBox2->vY.fY

				/' For both axis '/
				pvAxis = @(pstBox1->vX)
				j = 2
				While (j <> 0)							
					Dim As orxFLOAT  fMin
					Dim As orxFLOAT  fMax
					Dim As orxFLOAT  fProj
					Dim As orxFLOAT  orxU32    k

					/' Gets initial projected values '/
					orxVector_2DDot(@vToCorner[0], pvAxis,fProj)
					fMin = fProj:fMax = fProj

					/' For all remaining corners '/
					For k = 1 To 3
					
						/' Gets projected value '/
						orxVector_2DDot(@vToCorner[k], pvAxis,fProj)

						/' Updates extrema '/
						If(fProj > fMax) Then
						
							fMax = fProj
						ElseIf(fProj < fMin) Then
						
							fMin = fProj
						End If
					Next k

					/' Not intersecting? '/
					Dim As orxFLOAT fTEMP
					orxVector_GetSquareSize(pvAxis,fTEMP)
					If((fMax < orxFLOAT_0) OrElse (fMin > fTEMP)) Then
						/' Updates result '/
						_result = orxFALSE
						Exit For
					End If
					j -= 1
					pvAxis += 1
				Wend
				i -= 1:pstTemp = pstBox1:pstBox1 = pstBox2:pstBox2 = pstTemp:pvTemp = pvOrigin1:pvOrigin1 = pvOrigin2:pvOrigin2 = pvTemp
			Wend  
		Else  
			// Updates result
			_result = orxFALSE
		End If

		/' Done! '/
End Scope

#endmacro


#endif /' _orxOBOX_BI '/

/'* @} '/
