/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxVector.h
 * @date 30/03/2005
 * @author iarwain@orx-project.org
 *
 * @todo
 * - Add rotate function with matrix module, when it's done
 '/

/'*
 * @addtogroup orxVector
 * 
 * Vector module
 * Module that handles vectors and basic structures based on them
 *
 * @{
 '/


#ifndef _orxVECTOR_BI
#define _orxVECTOR_BI

#include "orxInclude.bi"
#include "orx/debug/orxDebug.bi"
#include "orx/memory/orxMemory.bi"
#include "orx/math/orxMath.bi"


/'* Public structure definition
 '/
Type __orxVECTOR_t

  /'* Coordinates : 12 '/
  Union  
    As orxFLOAT fX              /'*< First coordinate in the cartesian space '/
    As orxFLOAT fRho            /'*< First coordinate in the spherical space '/
    As orxFLOAT fR              /'*< First coordinate in the RGB color space '/
  End Union

  Union  
    As orxFLOAT fY              /'*< Second coordinate in the cartesian space '/
    As orxFLOAT fTheta          /'*< Second coordinate in the spherical space '/
    As orxFLOAT fG              /'*< Second coordinate in the RGB color space '/
  End Union

  Union
    As orxFLOAT fZ              /'*< Third coordinate in the cartesian space '/
    As orxFLOAT fPhi            /'*< Third coordinate in the spherical space '/
    As orxFLOAT fB              /'*< Third coordinate in the RGB color space '/
  End Union

End Type

Type orxVECTOR As __orxVECTOR_t

/' *** Vector inlined functions *** '/


/'* Sets vector XYZ values (also work for other coordinate system)
 * @param[in]   _pvVec                        Concerned vector
 * @param[in]   _fX                           First coordinate value
 * @param[in]   _fY                           Second coordinate value
 * @param[in]   _fZ                           Third coordinate value
 * @return      Vector
 '/
#macro orxVector_Set(_pvVec,_fX,_fY,_fZ,_result)

  /' Checks '/
  orxASSERT(_pvVec <> orxNULL)

  /' Stores values '/
  _pvVec->fX = _fX
  _pvVec->fY = _fY
  _pvVec->fZ = _fZ

  /' Done ! '/
  _result = _pvVec

#endmacro

/'* Sets all the vector coordinates with the given value
 * @param[in]   _pvVec                        Concerned vector
 * @param[in]   _fValue                       Value to set
 * @return      Vector
 '/
#macro orxVector_SetAll(_pvVec,_fValue,_result)

  /' Done ! '/
  orxVector_Set(_pvVec, _fValue, _fValue, _fValue,_result)

#endmacro

/'* Copies a vector onto another one
 * @param[in]   _pvDst                        Vector to copy to (destination)
 * @param[in]   _pvSrc                        Vector to copy from (source)
 * @return      Destination vector
 '/
#macro orxVector_Copy(_pvDst,_pvSrc,_result)

  /' Checks '/
  orxASSERT(_pvDst <> orxNULL)
  orxASSERT(_pvSrc <> orxNULL)

  /' Copies it '/
  orxMemory_Copy(_pvDst, _pvSrc, SizeOf(orxVECTOR))

  /' Done! '/
  _result = _pvDst

#endmacro

/'* Adds vectors and stores result in a third one
 * @param[out]  _pvRes                        Vector where to store result (can be one of the two operands)
 * @param[in]   _pvOp1                        First operand
 * @param[in]   _pvOp2                        Second operand
 * @return      Resulting vector (Op1 + Op2)
 '/
#macro orxVector_Add(_pvRes,_pvOp1,_pvOp2,_result)

  /' Checks '/
  orxASSERT(_pvRes <> orxNULL)
  orxASSERT(_pvOp1 <> orxNULL)
  orxASSERT(_pvOp2 <> orxNULL)

  /' Adds all '/
  _pvRes->fX = _pvOp1->fX + _pvOp2->fX
  _pvRes->fY = _pvOp1->fY + _pvOp2->fY
  _pvRes->fZ = _pvOp1->fZ + _pvOp2->fZ

  /' Done! '/
  _result = _pvRes
#endmacro

/'* Substracts vectors and stores result in a third one
 * @param[out]  _pvRes                        Vector where to store result (can be one of the two operands)
 * @param[in]   _pvOp1                        First operand
 * @param[in]   _pvOp2                        Second operand
 * @return      Resulting vector (Op1 - Op2)
 '/
#macro orxVector_Sub(_pvRes,_pvOp1,_pvOp2,_result)

  /' Checks '/
  orxASSERT(_pvRes <> orxNULL)
  orxASSERT(_pvOp1 <> orxNULL)
  orxASSERT(_pvOp2 <> orxNULL)

  /' Adds all '/
  _pvRes->fX = _pvOp1->fX - _pvOp2->fX
  _pvRes->fY = _pvOp1->fY - _pvOp2->fY
  _pvRes->fZ = _pvOp1->fZ - _pvOp2->fZ

  /' Done! '/
  _result = _pvRes

#endmacro

/'* Multiplies a vector by an orxFLOAT and stores result in another one
 * @param[out]  _pvRes                        Vector where to store result (can be the operand)
 * @param[in]   _pvOp1                        First operand
 * @param[in]   _fOp2                         Second operand
 * @return      Resulting vector
 '/
#macro orxVector_Mulf(_pvRes,_pvOp1,_fOp2,_result)

  /' Checks '/
  orxASSERT(_pvRes <> orxNULL)
  orxASSERT(_pvOp1 <> orxNULL)

  /' Muls all '/
  _pvRes->fX = _pvOp1->fX * _fOp2
  _pvRes->fY = _pvOp1->fY * _fOp2
  _pvRes->fZ = _pvOp1->fZ * _fOp2

  /' Done! '/
  _result = _pvRes

#endmacro

/'* Multiplies a vector by another vector and stores result in a third one
 * @param[out]  _pvRes                        Vector where to store result (can be one of the two operands)
 * @param[in]   _pvOp1                        First operand
 * @param[in]   _pvOp2                        Second operand
 * @return      Resulting vector (Op1 * Op2)
 '/
#macro orxVector_Mul(_pvRes,_pvOp1,_pvOp2,_result)

  /' Checks '/
  orxASSERT(_pvRes <> orxNULL)
  orxASSERT(_pvOp1 <> orxNULL)
  orxASSERT(_pvOp2 <> orxNULL)

  /' Muls all '/
  _pvRes->fX = _pvOp1->fX * _pvOp2->fX
  _pvRes->fY = _pvOp1->fY * _pvOp2->fY
  _pvRes->fZ = _pvOp1->fZ * _pvOp2->fZ

  /' Done! '/
  _result = _pvRes

#endmacro

/'* Divides a vector by an orxFLOAT and stores result in another one
 * @param[out]  _pvRes                        Vector where to store result (can be the operand)
 * @param[in]   _pvOp1                        First operand
 * @param[in]   _fOp2                         Second operand
 * @return      Resulting vector
 '/
#macro orxVector_Divf(_pvRes,_pvOp1,_fOp2,_result)

  
Scope 
		Dim As orxFLOAT fRecCoef
		/' Checks '/
		orxASSERT(_pvRes <> orxNULL)
		orxASSERT(_pvOp1 <> orxNULL)
		orxASSERT(_fOp2 <> orxFLOAT_0)
		/' Gets reciprocal coef '/
  	fRecCoef = orxFLOAT_1 / _fOp2
		/' Muls all '/
		_pvRes->fX = _pvOp1->fX * fRecCoef
		_pvRes->fY = _pvOp1->fY * fRecCoef
		_pvRes->fZ = _pvOp1->fZ * fRecCoef
	/' Done! '/
	_result = _pvRes	
End Scope

#endmacro

/'* Divides a vector by another vector and stores result in a third one
 * @param[out]  _pvRes                        Vector where to store result (can be one of the two operands)
 * @param[in]   _pvOp1                        First operand
 * @param[in]   _pvOp2                        Second operand
 * @return      Resulting vector (Op1 / Op2)
 '/
#macro orxVector_Div(_pvRes,_pvOp1,_pvOp2,_result)

  /' Checks '/
  orxASSERT(_pvRes <> orxNULL)
  orxASSERT(_pvOp1 <> orxNULL)
  orxASSERT(_pvOp2 <> orxNULL)
  orxASSERT(_pvOp2->fX <> orxFLOAT_0)
  orxASSERT(_pvOp2->fY <> orxFLOAT_0)
  orxASSERT(_pvOp2->fZ <> orxFLOAT_0)

  /' Divs all '/
  _pvRes->fX = _pvOp1->fX / _pvOp2->fX
  _pvRes->fY = _pvOp1->fY / _pvOp2->fY
  _pvRes->fZ = _pvOp1->fZ / _pvOp2->fZ

  /' Done! '/
  _result = _pvRes

#endmacro

/'* Lerps from one vector to another one using a coefficient
 * @param[out]  _pvRes                        Vector where to store result (can be one of the two operands)
 * @param[in]   _pvOp1                        First operand
 * @param[in]   _pvOp2                        Second operand
 * @param[in]   _fOp                          Lerp coefficient parameter
 * @return      Resulting vector
 '/
#macro orxVector_Lerp(_pvRes,_pvOp1,_pvOp2,_fOp,_result)

  /' Checks '/
  orxASSERT(_pvRes <> orxNULL)
  orxASSERT(_pvOp1 <> orxNULL)
  orxASSERT(_pvOp2 <> orxNULL)
  orxASSERT(_fOp >= orxFLOAT_0)

  /' Lerps all'/
  _pvRes->fX = orxLERP(_pvOp1->fX, _pvOp2->fX, _fOp)
  _pvRes->fY = orxLERP(_pvOp1->fY, _pvOp2->fY, _fOp)
  _pvRes->fZ = orxLERP(_pvOp1->fZ, _pvOp2->fZ, _fOp)

  /' Done! '/
  _result = _pvRes

#endmacro

/'* Gets minimum between two vectors
 * @param[out]  _pvRes                        Vector where to store result (can be one of the two operands)
 * @param[in]   _pvOp1                        First operand
 * @param[in]   _pvOp2                        Second operand
 * @return      Resulting vector MIN(Op1, Op2)
 '/
#macro orxVector_Min(_pvRes,_pvOp1,_pvOp2,_result)

  /' Checks '/
  orxASSERT(_pvRes <> orxNULL)
  orxASSERT(_pvOp1 <> orxNULL)
  orxASSERT(_pvOp2 <> orxNULL)

  /' Gets all mins '/
  _pvRes->fX = orxMIN(_pvOp1->fX, _pvOp2->fX)
  _pvRes->fY = orxMIN(_pvOp1->fY, _pvOp2->fY)
  _pvRes->fZ = orxMIN(_pvOp1->fZ, _pvOp2->fZ)

  /' Done! '/
  _result = _pvRes

#endmacro

/'* Gets maximum between two vectors
 * @param[out]  _pvRes                        Vector where to store result (can be one of the two operands)
 * @param[in]   _pvOp1                        First operand
 * @param[in]   _pvOp2                        Second operand
 * @return      Resulting vector MAX(Op1, Op2)
 '/
#macro orxVector_Max(_pvRes,_pvOp1,_pvOp2,_result)

  /' Checks '/
  orxASSERT(_pvRes <> orxNULL)
  orxASSERT(_pvOp1 <> orxNULL)
  orxASSERT(_pvOp2 <> orxNULL)

  /' Gets all maxs '/
  _pvRes->fX = orxMAX(_pvOp1->fX, _pvOp2->fX)
  _pvRes->fY = orxMAX(_pvOp1->fY, _pvOp2->fY)
  _pvRes->fZ = orxMAX(_pvOp1->fZ, _pvOp2->fZ)

  /' Done! '/
  _result = _pvRes

#endmacro

/'* Clamps a vector between two others
 * @param[out]  _pvRes                        Vector where to store result (can be the operand)
 * @param[in]   _pvOp                         Vector to clamp
 * @param[in]   _pvMin                        Minimum boundary
 * @param[in]   _pvMax                        Maximum boundary
 * @return      Resulting vector CLAMP(Op, MIN, MAX)
 '/
#macro orxVector_Clamp(_pvRes,_pvOp,_pvMin,_pvMax,_result)

  /' Checks '/
  orxASSERT(_pvRes <> orxNULL)
  orxASSERT(_pvOp <> orxNULL)
  orxASSERT(_pvMin <> orxNULL)
  orxASSERT(_pvMax <> orxNULL)

  /' Gets all clamped values '/
  _pvRes->fX = orxCLAMP(_pvOp->fX, _pvMin->fX, _pvMax->fX)
  _pvRes->fY = orxCLAMP(_pvOp->fY, _pvMin->fY, _pvMax->fY)
  _pvRes->fZ = orxCLAMP(_pvOp->fZ, _pvMin->fZ, _pvMax->fZ)

  /' Done! '/
  _result = _pvRes

#endmacro

/'* Negates a vector and stores result in another one
 * @param[out]  _pvRes                        Vector where to store result (can be the operand)
 * @param[in]   _pvOp                         Vector to negates
 * @return      Resulting vector (-Op)
 '/
#macro orxVector_Neg(_pvRes,_pvOp,_result)

  /' Checks '/
  orxASSERT(_pvRes <> orxNULL)
  orxASSERT(_pvOp <> orxNULL)

  /' Negates all '/
  _pvRes->fX = -(_pvOp->fX)
  _pvRes->fY = -(_pvOp->fY)
  _pvRes->fZ = -(_pvOp->fZ)

  /' Done! '/
  _result = _pvRes

#endmacro

/'* Gets reciprocal (1.0 /) vector and stores the result in another one
 * @param[out]  _pvRes                        Vector where to store result (can be the operand)
 * @param[in]   _pvOp                         Input value
 * @return      Resulting vector (1 / Op)
 '/
#macro orxVector_Rec(_pvRes,_pvOp,_result)

  /' Checks '/
  orxASSERT(_pvRes <> orxNULL)
  orxASSERT(_pvOp <> orxNULL)

  /' Reverts all '/
  _pvRes->fX = orxFLOAT_1 / _pvOp->fX
  _pvRes->fY = orxFLOAT_1 / _pvOp->fY
  _pvRes->fZ = orxFLOAT_1 / _pvOp->fZ

  /' Done! '/
  _result = _pvRes

#endmacro

/'* Gets floored vector and stores the result in another one
 * @param[out]  _pvRes                        Vector where to store result (can be the operand)
 * @param[in]   _pvOp                         Input value
 * @return      Resulting vector Floor(Op)
 '/
#macro orxVector_Floor(_pvRes,_pvOp,_result)

  /' Checks '/
  orxASSERT(_pvRes <> orxNULL)
  orxASSERT(_pvOp <> orxNULL)

  /' Reverts all '/
  _pvRes->fX = orxMath_Floor(_pvOp->fX)
  _pvRes->fY = orxMath_Floor(_pvOp->fY)
  _pvRes->fZ = orxMath_Floor(_pvOp->fZ)

  /' Done! '/
  _result = _pvRes

#endmacro

/'* Gets rounded vector and stores the result in another one
 * @param[out]  _pvRes                        Vector where to store result (can be the operand)
 * @param[in]   _pvOp                         Input value
 * @return      Resulting vector Round(Op)
 '/
#macro orxVector_Round(_pvRes,_pvOp,_result)

  /' Checks '/
  orxASSERT(_pvRes <> orxNULL)
  orxASSERT(_pvOp <> orxNULL)

  /' Reverts all '/
  _pvRes->fX = orxMath_Round(_pvOp->fX)
  _pvRes->fY = orxMath_Round(_pvOp->fY)
  _pvRes->fZ = orxMath_Round(_pvOp->fZ)

  /' Done! '/
  _result = _pvRes

#endmacro

/'* Gets vector squared size
 * @param[in]   _pvOp                         Input vector
 * @return      Vector's squared size
 '/
#macro   orxVector_GetSquareSize(_pvOp,_result)  

  /' Checks '/
  orxASSERT(_pvOp <> orxNULL)

  /' Updates result '/
  _result = = (_pvOp->fX * _pvOp->fX) + (_pvOp->fY * _pvOp->fY) + (_pvOp->fZ * _pvOp->fZ)

  /' Done! '/
  
#endmacro

/'* Gets vector size
 * @param[in]   _pvOp                         Input vector
 * @return      Vector's size
 '/
#macro orxVector_GetSize(_pvOp,_result)
 

  /' Checks '/
  orxASSERT(_pvOp <> orxNULL)

  /' Updates result '/
  orxMath_Sqrt((_pvOp->fX * _pvOp->fX) + (_pvOp->fY * _pvOp->fY) + (_pvOp->fZ * _pvOp->fZ),_result)

  /' Done! '/
  
#endmacro

/'* Gets squared distance between 2 positions
 * @param[in]   _pvOp1                        First position
 * @param[in]   _pvOp2                        Second position
 * @return      Squared distance
 '/
#macro orxVector_GetSquareDistance(_pvOp1,_pvOp2,_result)
  
Scope
		Dim As orxVECTOR   vTemp
    Dim As orxVECTOR   vTemp2
		/' Checks '/
		orxASSERT(_pvOp1 <> orxNULL)
		orxASSERT(_pvOp2  <> orxNULL)

		/' Gets distance vector '/
		orxVector_Sub(@vTemp, _pvOp2, _pvOp1,vTemp2)  
		/' Updates result '/
		orxVector_GetSquareSize(@vTemp,_result)
		/' Done! '/
End Scope
  
#endmacro

/'* Gets distance between 2 positions
 * @param[in]   _pvOp1                        First position
 * @param[in]   _pvOp2                        Second position
 * @return      Distance
 '/
#macro orxVector_GetDistance(_pvOp1,_pvOp2,_result)

Scope
		Dim As orxVECTOR   vTemp
    Dim As orxVECTOR   vTemp2
		
		/' Checks '/
		orxASSERT(_pvOp1 <> orxNULL)
		orxASSERT(_pvOp2  <> orxNULL)

		/' Gets distance vector '/
		orxVector_Sub(@vTemp, _pvOp2, _pvOp1,@vTemp2)

		/' Updates result '/
		orxVector_GetSize(@vTemp,_result)

		/' Done! '/
End Scope
  
#endmacro

/'* Normalizes a vector
 * @param[out]  _pvRes                        Vector where to store result (can be the operand)
 * @param[in]   _pvOp                         Vector to normalize
 * @return      Normalized vector
 '/
#macro orxVector_Normalize(_pvRes,_pvOp,_result)

Scope
		Dim As orxFLOAT fOp
    Dim As orxFLOAT fResult
		/' Checks '/
		orxASSERT(_pvRes <> orxNULL)
		orxASSERT(_pvOp <> orxNULL)

		/' Gets squared size '/
		fOp = (_pvOp->fX * _pvOp->fX) + (_pvOp->fY * _pvOp->fY) + (_pvOp->fZ * _pvOp->fZ)

		/' Valid? '/
		If(fOp > orxFLOAT_0) Then		
			/' Gets reciprocal size '/
			orxMath_Sqrt(fOp,fResult)
			fOp = orxFLOAT_1 / fResult

			/' Updates result '/
			_pvRes->fX = fOp * _pvOp->fX
			_pvRes->fY = fOp * _pvOp->fY
			_pvRes->fZ = fOp * _pvOp->fZ
		
		Else
		
			/' Clears result '/
			orxMemory_Zero(_pvRes, SizeOf(orxVECTOR))
		End If

		/' Done! '/
		_result = _pvRes
End Scope

#endmacro

/'* Rotates a 2D vector (along Z-axis)
 * @param[out]  _pvRes                        Vector where to store result (can be the operand)
 * @param[in]   _pvOp                         Vector to rotate
 * @param[in]   _fAngle                       Angle of rotation (radians)
 * @return      Rotated vector
 '/
#macro orxVector_2DRotate(_pvRes,_pvOp,_fAngle,_result)

Scope
	
		Dim As orxFLOAT fSin
		Dim As orxFLOAT fCos
    Dim As orxVECTOR vTemp
		
		/' Checks '/
		orxASSERT(_pvRes   <> orxNULL)
		orxASSERT(_pvOp <> orxNULL)

		/' Gets cos & sin of angle '/
		orxMath_Cos(_fAngle,fCos)
		orxMath_Sin(_fAngle,fSin)

		/' Updates result '/
		orxVector_Set(_pvRes, (fCos * _pvOp->fX) - (fSin * _pvOp->fY), (fSin * _pvOp->fX) + (fCos * _pvOp->fY), _pvOp->fZ,vTemp)

		/' Done! '/
		_result = _pvRes

End Scope

#endmacro

/'* Is vector null?
 * @param[in]   _pvOp                         Vector to test
 * @return      orxTRUE if vector's null, orxFALSE otherwise
 '/
#macro orxVector_IsNull(_pvOp,_result)

  

  /' Checks '/
  orxASSERT(_pvOp <> orxNULL)

  /' Updates result '/
  _result = Iif(((_pvOp->fX = orxFLOAT_0) AndAlso (_pvOp->fY = orxFLOAT_0) AndAlso (_pvOp->fZ = orxFLOAT_0)),orxTRUE,orxFALSE)

  /' Done! '/
  
#endmacro

/'* Are vectors equal?
 * @param[in]   _pvOp1                        First vector to compare
 * @param[in]   _pvOp2                        Second vector to compare
 * @return      orxTRUE if both vectors are equal, orxFALSE otherwise
 '/
#macro orxVector_AreEqual(_pvOp1,_pvOp2,_result)

  

  /' Checks '/
  orxASSERT(_pvOp1  <> orxNULL)
  orxASSERT(_pvOp2  <> orxNULL)

  /' Updates result '/
  _result = (((_pvOp1->fX = _pvOp2->fX) AndAlso (_pvOp1->fY = _pvOp2->fY) AndAlso (_pvOp1->fZ = _pvOp2->fZ)), orxTRUE,orxFALSE)

  /' Done! '/
  
#endmacro

/'* Transforms a cartesian vector into a spherical one
 * @param[out]  _pvRes                        Vector where to store result (can be the operand)
 * @param[in]   _pvOp                         Vector to transform
 * @return      Transformed vector
 '/
#macro orxVector_FromCartesianToSpherical(_pvRes,_pvOp,_result)

Scope
  /' Checks '/
  orxASSERT(_pvRes <> orxNULL)
  orxASSERT(_pvOp <> orxNULL)

  /' Is operand vector null? '/
  If((_pvOp->fX == orxFLOAT_0) AndAlso (_pvOp->fY == orxFLOAT_0) AndAlso (_pvOp->fZ == orxFLOAT_0)) Then
  
    /' Updates result vector '/
    _pvRes->fRho = _pvRes->fTheta = _pvRes->fPhi = orxFLOAT_0
  
  Else
  
    Dim As orxFLOAT fRho
		Dim As orxFLOAT fTheta
		Dim As orxFLOAT fPhi

    /' Z = 0? '/
    If(_pvOp->fZ = orxFLOAT_0) Then    
      /' X = 0? '/
      If(_pvOp->fX = orxFLOAT_0) Then      
        /' Gets absolute value '/
        orxMath_Abs(_pvOp->fY,fRho)
      /' X <> 0 and Y = 0? '/
      ElseIf(_pvOp->fY = orxFLOAT_0) Then      
        /' Gets absolute value '/
        orxMath_Abs(_pvOp->fX,fRho)
      /' X <> 0 and Y <> 0 '/
      Else      
        /' Computes rho '/
        orxMath_Sqrt((_pvOp->fX * _pvOp->fX) + (_pvOp->fY * _pvOp->fY),fRho)
      End If
      /' Sets phi '/
      fPhi = orxMATH_KF_PI_BY_2
    Else    
      /' X = 0 and Y = 0? '/
      If((_pvOp->fX = orxFLOAT_0) AndAlso (_pvOp->fY = orxFLOAT_0)) Then
              /' Z < 0? '/
        If(_pvOp->fZ < orxFLOAT_0) Then        
          /' Gets absolute value '/
          orxMath_Abs(_pvOp->fZ,fRho)

          /' Sets phi '/
          fPhi = orxMATH_KF_PI   
        Else        
          /' Sets rho '/
          fRho = _pvOp->fZ

          /' Sets phi '/
          fPhi = orxFLOAT_0
        End If      
      Else      
        /' Computes rho '/
        Dim As orxFLOAT fTEMP
				orxVector_GetSquareSize(_pvOp,fTEMP)
				orxMath_Sqrt(fTEMP,fRho)

        /' Computes phi '/
        orxMath_ACos(_pvOp->fZ / fRho,fPhi)
      End If
    End If

    /' Computes theta '/
    orxMath_ATan(_pvOp->fY, _pvOp->fX,fTheta)

    /' Updates result '/
    _pvRes->fRho    = fRho
    _pvRes->fTheta  = fTheta
    _pvRes->fPhi    = fPhi
 End  If
  /' Done! '/
  _result = _pvRes

End Scope
#endmacro

/'* Transforms a spherical vector into a cartesian one
 * @param[out]  _pvRes                        Vector where to store result (can be the operand)
 * @param[in]   _pvOp                         Vector to transform
 * @return      Transformed vector
 '/
#macro orxVector_FromSphericalToCartesian(_pvRes,_pvOp,_result)

Scope
		Dim As orxFLOAT fSinPhi
		Dim As orxFLOAT fCosPhi
		Dim As orxFLOAT fSinTheta
		Dim As orxFLOAT fCosTheta
		Dim As orxFLOAT fRho
    Dim As orxFLOAT fTmp
		
		/' Checks '/
		orxASSERT(_pvRes <> orxNULL)
		orxASSERT(_pvOp <> orxNULL)

		/' Stores rho '/
		fRho = _pvOp->fRho

		/' Gets sine & cosine '/
		orxMath_Sin(_pvOp->fTheta,fSinTheta)
		orxMath_Cos(_pvOp->fTheta,fCosTheta)
		orxMath_Sin(_pvOp->fPhi,fSinPhi)
		orxMath_Cos(_pvOp->fPhi,fCosPhi)
		orxMath_Abs(fSinTheta,fTmp)
		If(fTmp < orxMATH_KF_EPSILON) Then		
			fSinTheta = orxFLOAT_0
		End If
		orxMath_Abs(fCosTheta,fTmp)
		If(fTmp < orxMATH_KF_EPSILON) Then		
			fCosTheta = orxFLOAT_0
		End If
		orxMath_Abs(fSinPhi,fTmp)
		If(fTmp < orxMATH_KF_EPSILON) Then		
			fSinPhi = orxFLOAT_0
		End If
		orxMath_Abs(fCosPhi,fTmp)
		If(fTmp < orxMATH_KF_EPSILON) Then		
			fCosPhi = orxFLOAT_0
		End If
		
		/' Updates result '/
		_pvRes->fX = fRho * fCosTheta * fSinPhi
		_pvRes->fY = fRho * fSinTheta * fSinPhi
		_pvRes->fZ = fRho * fCosPhi

		/' Done! '/
		_result = _pvRes
End Scope

#endmacro

/'* Gets dot product of two vectors
 * @param[in]   _pvOp1                      First operand
 * @param[in]   _pvOp2                      Second operand
 * @return      Dot product
 '/
#macro orxVector_Dot(_pvOp1,_pvOp2,_result)

  /' Checks '/
  orxASSERT(_pvOp1 <> orxNULL)
  orxASSERT(_pvOp2  <> orxNULL)

  /' Updates result '/
  _result = (_pvOp1->fX * _pvOp2->fX) + (_pvOp1->fY * _pvOp2->fY) + (_pvOp1->fZ * _pvOp2->fZ)

  /' Done! '/
  
#endmacro

/'* Gets 2D dot product of two vectors
 * @param[in]   _pvOp1                      First operand
 * @param[in]   _pvOp2                      Second operand
 * @return      2D dot product
 '/
#macro orxVector_2DDot(_pvOp1,_pvOp2,_result)
  

  /' Checks '/
  orxASSERT(_pvOp1 <> orxNULL)
  orxASSERT(_pvOp2  <> orxNULL)

  /' Updates result '/
  _result = (_pvOp1->fX * _pvOp2->fX) + (_pvOp1->fY * _pvOp2->fY)

  /' Done! '/
  
#endmacro

/'* Gets cross product of two vectors
 * @param[out]  _pvRes                       Vector where to store result
 * @param[in]   _pvOp1                      First operand
 * @param[in]   _pvOp2                      Second operand
 * @return      Cross product orxVECTOR / orxNULL
 '/
#macro orxVector_Cross(_pvRes,_pvOp1,_pvOp2,_result)  
Scope
	
		Dim As orxFLOAT fTemp1
		Dim As orxFLOAT fTemp2

		/' Checks '/
		orxASSERT(_pvRes <> orxNULL)
		orxASSERT(_pvOp1 <> orxNULL)
		orxASSERT(_pvOp2  <> orxNULL)

		/' Computes cross product '/
		fTemp1      = (_pvOp1->fY * _pvOp2->fZ) - (_pvOp1->fZ * _pvOp2->fY)
		fTemp2      = (_pvOp1->fZ * _pvOp2->fX) - (_pvOp1->fX * _pvOp2->fZ)
		_pvRes->fZ  = (_pvOp1->fX * _pvOp2->fY) - (_pvOp1->fY * _pvOp2->fX)
		_pvRes->fY  = fTemp2
		_pvRes->fX  = fTemp1

		/' Done! '/
		_result = _pvRes
	
End Scope

#endmacro


/' *** Vector constants *** '/

Extern orxVECTOR_X As orxVECTOR       /'*< X-Axis unit vector '/
Extern orxVECTOR_Y As orxVECTOR       /'*< Y-Axis unit vector '/
Extern orxVECTOR_Z As orxVECTOR      /'*< Z-Axis unit vector '/

Extern orxVECTOR_0 As orxVECTOR     /'*< Null vector '/
Extern orxVECTOR_1 As orxVECTOR     /'*< Vector filled with 1s '/

Extern orxVECTOR_RED   As orxVECTOR  /'*< Red color vector '/
Extern orxVECTOR_GREEN As orxVECTOR  /'*< Green color vector '/
Extern orxVECTOR_BLUE  As orxVECTOR  /'*< Blue color vector '/
Extern orxVECTOR_BLACK As orxVECTOR  /'*< Black color vector '/
Extern orxVECTOR_WHITE As orxVECTOR  /'*< White color vector '/

#endif /' _orxVECTOR_BI '/

/'* @} '/
