/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxFrame.h
 * @date 01/12/2003
 * @author iarwain@orx-project.org
 *
 * @todo
 * - Use matrix instead of disjoint position vector/angle float/scale vector
 * - Add 3D support
 '/

/'*
 * @addtogroup orxFrame
 *
 * Frame (scene node) Module.
 * Allows to handles frame (scene nodes).
 * It consists in a nodes (arranged in a hierarchical tree)
 * having their position given in their parent local coordinate subsystem
 * and caching their global one (in the root coordinate system).
 * They're used to give position, rotation & scale to an object or a group of objects.
 * Frames are structures.
 * They thus can be referenced by Object Module.
 *
 * @{
 '/


#ifndef _orxFRAME_BI
#define _orxFRAME_BI

#include "orxInclude.bi"
#include "orx/math/orxVector.bi"


/'* Frame flags
 '/
#define orxFRAME_KU32_FLAG_NONE               &H00000000  /'*< No flags '/

#define orxFRAME_KU32_FLAG_SCROLL_X           &H00000001  /'*< X axis differential scrolling flag '/
#define orxFRAME_KU32_FLAG_SCROLL_Y           &H00000002  /'*< Y axis differential scrolling flag '/
#define orxFRAME_KU32_MASK_SCROLL_BOTH        &H00000003  /'*< Both axis differential scrolling mask '/

#define orxFRAME_KU32_FLAG_DEPTH_SCALE        &H00000004  /'*< Relative depth scaling flag '/

#define orxFRAME_KU32_FLAG_FLIP_X             &H00000010  /'*< X axis flipping flag '/
#define orxFRAME_KU32_FLAG_FLIP_Y             &H00000020  /'*< Y axis flipping flag '/
#define orxFRAME_KU32_MASK_FLIP_BOTH          &H00000030  /'*< Both axis flippinf mask '/

#define orxFRAME_KU32_MASK_USER_ALL           &H000000FF  /'*< User all ID mask '/


/'* Frame space enum
 '/
Enum orxFRAME_SPACE

  orxFRAME_SPACE_GLOBAL = 0,
  orxFRAME_SPACE_LOCAL,

  orxFRAME_SPACE_NUMBER,

  orxFRAME_SPACE_NONE = orxENUM_NONE

End Enum



/'* Internal Frame structure
 '/
Type __orxFRAME_t As Any
Type orxFRAME As __orxFRAME_t

Extern "Windows"

/'* Setups the frame module
 '/
Declare Sub orxFrame_Setup()

/'* Inits the frame module
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxFrame_Init() As orxSTATUS

/'* Exits from the frame module
 '/
Declare Sub orxFrame_Exit()


/'* Creates a frame
 * @param[in]   _u32Flags     flags for created animation
 * @return      Created orxFRAME / orxNULL
 '/
Declare Function orxFrame_Create(ByVal _u32Flags As orxU32) As orxFRAME Ptr

/'* Deletes a frame
 * @param[in]   _pstFrame       Frame to delete
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxFrame_Delete(ByVal _pstFrame As orxFRAME Ptr) As orxSTATUS


/'* Sets a frame parent
 * @param[in]   _pstFrame       Concerned frame
 * @param[in]   _pstParent      Parent frame to set
 '/
Declare Sub orxFrame_SetParent(ByVal _pstFrame As orxFRAME Ptr,ByVal _pstParent As orxFRAME Ptr)

/'* Is a root child?
 * @param[in]   _pstFrame       Concerned frame
 * @return orxTRUE if its parent is root, orxFALSE otherwise
 '/
Declare Function orxFrame_IsRootChild(ByVal _pstFrame As Const orxFRAME Ptr) As orxBOOL


/'* Is dirty?
 * @param[in]   _pstFrame       Concerned frame
 * @return orxTRUE if dirty, orxFALSE otherwise
 '/
Declare Function orxFrame_IsDirty(ByVal _pstFrame As Const orxFRAME Ptr) As orxBOOL


/'* Sets a frame position
 * @param[in]   _pstFrame       Concerned frame
 * @param[in]   _eSpace         Coordinate space system to use
 * @param[in]   _pvPos          Position to set
 '/
Declare Sub orxFrame_SetPosition(ByVal _pstFrame As orxFRAME Ptr,ByVal _eSpace As orxFRAME_SPACE,ByVal _pvPos As Const orxVECTOR Ptr)

/'* Sets a frame rotation
 * @param[in]   _pstFrame       Concerned frame
 * @param[in]   _eSpace         Coordinate space system to use
 * @param[in]   _fAngle         Angle to set (radians)
 '/
Declare Sub orxFrame_SetRotation(ByVal _pstFrame As orxFRAME Ptr,ByVal _eSpace As orxFRAME_SPACE,ByVal _fAngle As orxFLOAT)

/'* Sets a frame scale
 * @param[in]   _pstFrame       Concerned frame
 * @param[in]   _eSpace         Coordinate space system to use
 * @param[in]   _pvScale        Scale to set
 '/
Declare Sub orxFrame_SetScale(ByVal _pstFrame As orxFRAME Ptr,ByVal _eSpace As orxFRAME_SPACE,ByVal _pvScale As Const orxVECTOR Ptr)


/'* Gets a frame position
 * @param[in]   _pstFrame       Concerned frame
 * @param[in]   _eSpace         Coordinate space system to use
 * @param[out]  _pvPos          Position of the given frame
 * @return orxVECTOR / orxNULL
 '/
Declare Function orxFrame_GetPosition(ByVal _pstFrame As orxFRAME Ptr,ByVal _eSpace As orxFRAME_SPACE,ByVal _pvPos As orxVECTOR Ptr) As orxVECTOR Ptr

/'* Gets a frame rotation
 * @param[in]   _pstFrame       Concerned frame
 * @param[in]   _eSpace         Coordinate space system to use
 * @return Rotation of the given frame (radians)
 '/
Declare Function orxFrame_GetRotation(ByVal _pstFrame As orxFRAME Ptr,ByVal _eSpace As orxFRAME_SPACE) As orxFLOAT

/'* Gets a frame scale
 * @param[in]   _pstFrame       Concerned frame
 * @param[in]   _eSpace         Coordinate space system to use
 * @param[out]  _pvScale        Scale
 * @return      orxVECTOR / orxNULL
 '/
Declare Function orxFrame_GetScale(ByVal _pstFrame As orxFRAME Ptr,ByVal _eSpace As orxFRAME_SPACE,ByVal _pvScale As orxVECTOR Ptr) As orxVECTOR Ptr

End Extern

#endif /' _orxFRAME_bi '/

/'* @} '/
