/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxShader.h
 * @date 11/04/2009
 * @author iarwain@orx-project.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxShader
 *
 * Shader module
 * Allows to store shader information (code + parameters).
 *
 * @{
 '/


#ifndef _orxSHADER_BI
#define _orxSHADER_BI


#include "orxInclude.bi"
#include "orx/math/orxVector.bi"
#include "orx/display/orxTexture.bi"
#include "orx/object/orxObject.bi"
#include "orx/utils/orxLinkList.bi"


/'* Shader parameter type
 '/
Enum orxSHADER_PARAM_TYPE

  orxSHADER_PARAM_TYPE_FLOAT = 0,
  orxSHADER_PARAM_TYPE_TEXTURE,
  orxSHADER_PARAM_TYPE_VECTOR,

  orxSHADER_PARAM_TYPE_NUMBER,

  orxSHADER_PARAM_TYPE_NONE = orxENUM_NONE

End Enum


/'* Shader parameter structure
 '/
Type __orxSHADER_PARAM_t

  As orxLINKLIST_NODE      stNode                 /'*< Linklist node : 12 '/
  As orxSHADER_PARAM_TYPE  eType                  /'*< Parameter type : 16 '/
  As orxSTRING             zName                  /'*< Parameter literal name : 20 '/

End Type

Type orxSHADER_PARAM As __orxSHADER_PARAM_t


/'* Internal shader structure
 '/
Type orxSHADER As __orxSHADER_t


/'* Event enum
 '/
Enum orxSHADER_EVENT

  orxSHADER_EVENT_SET_PARAM = 0,                /'*< Event sent when setting a parameter '/

  orxSHADER_EVENT_NUMBER,

  orxSHADER_EVENT_NONE = orxENUM_NONE

End Enum


/'* Shader event payload
 '/
Type __orxSHADER_EVENT_PARAM_PAYLOAD_t

  As Const orxSHADER      Ptr pstShader              /'*< Shader reference : 4 '/
  As orxSTRING            zShaderName            /'*< Shader name : 8 '/

  As orxSHADER_PARAM_TYPE  eParamType             /'*< Parameter type : 12 '/
  As orxSTRING             zParamName             /'*< Parameter name : 16 '/

  Union  
    As orxFLOAT    fValue                         /'*< Float value : 20 '/
    As orxTEXTURE Ptr pstValue                       /'*< Texture value : 20 '/
    As orxVECTOR   vValue                         /'*< Vector value : 20 '/
  End Union

End Type

Type orxSHADER_EVENT_PARAM_PAYLOAD As __orxSHADER_EVENT_PARAM_PAYLOAD_t

Extern "Windows"

/'* Shader module setup
 '/
Declare Sub orxShader_Setup()

/'* Inits the shader module
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxShader_Init() As orxSTATUS

/'* Exits from the shader module
 '/
Declare Sub orxShader_Exit()

/'* Creates an empty shader
 * @return orxSHADER / orxNULL
 '/
Declare Function orxShader_Create() As orxSHADER Ptr

/'* Creates a shader from config
 * @param[in]   _zConfigID            Config ID
 * @ return orxSHADER / orxNULL
 '/
Declare Function orxShader_CreateFromConfig(ByVal _zConfigID As Const orxSTRING) As orxSHADER Ptr

/'* Deletes a shader
 * @param[in] _pstShader              Concerned Shader
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxShader_Delete(ByVal _pstShader As orxSHADER Ptr) As orxSTATUS

/'* Renders a shader
 * @param[in] _pstShader              Concerned Shader
 * @param[in] _pstOwner               Owner structure (orxOBJECT / orxVIEWPORT / orxNULL)
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxShader_Render(ByVal _pstShader As Const orxSHADER Ptr,ByVal _pstOwner As Const orxSTRUCTURE Ptr) As orxSTATUS


/'* Adds a float parameter definition to a shader (parameters need to be set before compiling the shader code)
 * @param[in] _pstShader              Concerned Shader
 * @param[in] _zName                  Parameter's literal name
 * @param[in] _fValue                 Parameter's float value
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxShader_AddFloatParam(ByVal _pstShader As orxSHADER Ptr,ByVal _zName As Const orxSTRING,ByVal _fValue As orxFLOAT) As orxSTATUS

/'* Adds a texture parameter definition to a shader (parameters need to be set before compiling the shader code)
 * @param[in] _pstShader              Concerned Shader
 * @param[in] _zName                  Parameter's literal name
 * @param[in] _pstValue               Parameter's texture value
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxShader_AddTextureParam(ByVal _pstShader As orxSHADER Ptr,ByVal _zName As Const orxSTRING,ByVal _pstValue As orxTEXTURE Ptr) As orxSTATUS

/'* Adds a vector parameter definition to a shader (parameters need to be set before compiling the shader code)
 * @param[in] _pstShader              Concerned Shader
 * @param[in] _zName                  Parameter's literal name
 * @param[in] _pvValue                Parameter's vector value
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxShader_AddVectorParam(ByVal _pstShader As orxSHADER Ptr,ByVal _zName As Const orxSTRING,ByVal _pvValue As Const orxVECTOR Ptr) As orxSTATUS

/'* Sets shader code & compiles it (parameters need to be set before compiling the shader code)
 * @param[in] _pstShader              Concerned Shader
 * @param[in] _zCode                  Shader's code to compile (parameters need to be set beforehand)
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxShader_CompileCode(ByVal _pstShader As orxSHADER Ptr,ByVal _zCode As Const orxSTRING) As orxSTATUS

/'* Gets shader parameter list
 * @param[in] _pstShader              Concerned Shader
 * @return orxLINKLIST / orxNULL
 '/
Declare Function orxShader_GetParamList(ByVal _pstShader As Const orxSHADER Ptr) As Const orxLINKLIST Ptr

/'* Enables/disables a shader
 * @param[in]   _pstShader            Concerned Shader
 * @param[in]   _bEnable              Enable / disable
 '/
Declare Sub orxShader_Enable(ByVal _pstShader As orxSHADER Ptr,ByVal _bEnable As orxBOOL)

/'* Is shader enabled?
 * @param[in]   _pstShader            Concerned Shader
 * @return      orxTRUE if enabled, orxFALSE otherwise
 '/
Declare Function orxShader_IsEnabled(ByVal _pstShader As Const orxSHADER Ptr) As orxBOOL

/'* Gets shader name
 * @param[in]   _pstShader            Concerned Shader
 * @return      orxSTRING / orxSTRING_EMPTY
 '/
Declare Function orxShader_GetName(ByVal _pstShader As Const orxSHADER Ptr) As Const orxSTRING

End Extern

#endif /' _orxSHADER_BI '/

/'* @} '/
