/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxLinkList.h
 * @date 06/04/2005
 * @author iarwain@orx-project.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxLinkList
 * 
 * Linklist module
 * Module that handles linklists
 *
 * @{
 *
 * @section linklist Link List - How to
 * This module provides an easy and powerful interface for manipulating linked lists.
 *
 * @subsection linklist_datadefine Data definition
 * Using this data structure as an example:
 * @code
 * typedef struct __orxFOO_t
 * {
 *   orxU32 u32Data;        Data
 * } orxFOO;
 * @endcode
 *
 * @subsection linklist_dataalloc Data without link
 * Creating a bank to allocate memory storage:
 * @code
 * orxBANK *pstBank = orxBank_Create(10, sizeof(orxFOO), orxBANK_KU32_FLAG_NONE, orxMEMORY_TYPE_MAIN);
 * @endcode
 * You can then instantiate it this way:
 * @code
 * orxFOO *pstNode = (orxFOO *)orxBank_Allocate(pstBank);
 * pstNode->u32Data = 205;
 * @endcode
 * Having this basic behavior, you can add list linking to it.
 * @subsection linklist_realalloc Linked list item definition
 * To do so, you need to include in your structure an orxLINKLIST_NODE member as *FIRST MEMBER*:
 * @code
 * typedef struct __orxFOO_t
 * {
 *  orxLINKLIST_NODE stNode;
 *  orxU32 u32Data;
 * } orxFOO;
 * @endcode
 * @subsection linklist_realuse Use of link list
 * Your data structure can now be linked in lists:
 * @code
 * orxLINKLIST stList;
 * orxLinkList_AddEnd(&stList, (orxLINKLIST_NODE *)pstNode);
 * @endcode
 * @note As the first member of your data structure is a linked list node, you can cast your structure to orxLINKLIST_NODE and reciprocally.
 '/


#ifndef _orxLINKLIST_BI
#define _orxLINKLIST_BI


#include "orxInclude.bi"
#include "orx/debug/orxDebug.bi"

Type orxLINKLIST As __orxLINKLIST_t

/'* Node list structure
 '/
Type __orxLINKLIST_NODE_t

  As __orxLINKLIST_NODE_t Ptr pstNext         /'*< Next node pointer : 4 '/
  As __orxLINKLIST_NODE_t Ptr pstPrevious   /'*< Previous node pointer : 8 '/
  As                  orxLINKLIST Ptr pstList          /'*< Associated list pointer : 12 '/

End Type

Type orxLINKLIST_NODE As __orxLINKLIST_NODE_t

/'* List structure
 '/
Type __orxLINKLIST_t

  As orxLINKLIST_NODE Ptr pstFirst                   /'*< First node pointer : 4 '/
  As orxLINKLIST_NODE Ptr pstLast                    /'*< Last node pointer : 8 '/
  As orxU32 u32Counter                    /'*< Node counter : 12 '/

End Type


Extern "Windows"

/'* Cleans a linklist
 * @param[in]   _pstList                        Concerned list
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxLinkList_Clean(ByVal _pstList As orxLINKLIST Ptr) As orxSTATUS


/'* Adds a node at the start of a list
 * @param[in]   _pstList                        Concerned list
 * @param[in]   _pstNode                        Node to add
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxLinkList_AddStart(ByVal _pstList As orxLINKLIST Ptr, ByVal _pstNode As orxLINKLIST_NODE Ptr) As orxSTATUS

/'* Adds a node at the end of a list
 * @param[in]   _pstList                        Concerned list
 * @param[in]   _pstNode                        Node to add
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxLinkList_AddEnd(ByVal _pstList As orxLINKLIST Ptr, ByVal _pstNode As orxLINKLIST_NODE Ptr) As orxSTATUS

/'* Adds a node before another one
 * @param[in]   _pstRefNode                     Reference node (add before this one)
 * @param[in]   _pstNode                        Node to add
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxLinkList_AddBefore(ByVal _pstRefNode As orxLINKLIST_NODE Ptr, ByVal _pstNode As orxLINKLIST_NODE Ptr) As orxSTATUS

/'* Adds a node after another one
 * @param[in]   _pstRefNode                     Reference node (add after this one)
 * @param[in]   _pstNode                        Node to add
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxLinkList_AddAfter(ByVal _pstRefNode As orxLINKLIST_NODE Ptr, ByVal _pstNode As orxLINKLIST_NODE Ptr) As orxSTATUS

/'* Removes a node from its list
 * @param[in]   _pstNode                        Concerned node
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxLinkList_Remove(ByVal _pstNode As orxLINKLIST_NODE Ptr) As orxSTATUS

End Extern

/' *** LinkList inlined accessors *** '/


/'* Gets a node list
 * @param[in]   _pstNode                        Concerned node
 * @return orxLINKLIST / orxNULL
 '/
#macro orxLinkList_GetList(_pstNode,_result)

  /' Checks '/
  orxASSERT(_pstNode <> orxNULL)

  /' Returns it '/
  _result = (_pstNode->pstList)
#endmacro

/'* Gets previous node in list
 * @param[in]   _pstNode                        Concerned node
 * @return orxLINKLIST_NODE / orxNULL
 '/
#macro orxLinkList_GetPrevious(_pstNode,_result)

  /' Checks '/
  orxASSERT(_pstNode <> orxNULL)

  /' Returns it '/
  _result = Iif((_pstNode->pstList <> orxNULL), _pstNode->pstPrevious,Cast(orxLINKLIST_NODE Ptr,orxNULL))
#endmacro

/'* Gets next node in list
 * @param[in]   _pstNode                        Concerned node
 * @return orxLINKLIST_NODE / orxNULL
 '/
#macro orxLinkList_GetNext(_pstNode,_result)

  /' Checks '/
  orxASSERT(_pstNode <> orxNULL)

  /' Returns it '/
  _result = Iif((_pstNode->pstList <> orxNULL), _pstNode->pstNext,Cast(orxLINKLIST_NODE Ptr,orxNULL))
#endmacro


/'* Gets a list first node
 * @param[in]   _pstList                        Concerned list
 * @return orxLINKLIST_NODE / orxNULL
 '/
#macro orxLinkList_GetFirst(_pstList,_result)

  /' Checks '/
  orxASSERT(_pstList <> orxNULL)

  /' Returns it '/
  _result = (_pstList->pstFirst)
#endmacro

/'* Gets a list last node
 * @param[in]   _pstList                        Concerned list
 * @return orxLINKLIST_NODE / orxNULL
 '/
#macro orxLinkList_GetLast(_pstList,_result)

  /' Checks '/
  orxASSERT(_pstList <> orxNULL)

  /' Returns it '/
  _result = (_pstList->pstLast)
#endmacro

/'* Gets a list counter
 * @param[in]   _pstList                        Concerned list
 * @return Number of nodes in list
 '/
#macro orxLinkList_GetCounter(_pstList,_result)

  /' Checks '/
  orxASSERT(_pstList <> orxNULL)

  /' Returns it '/
  _result = (_pstList->u32Counter)
#endmacro

#endif /' _orxLINKLIST_BI '/

/'* @} '/
