/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxString.h
 * @date 21/04/2005
 * @author bestel@arcallians.org
 *
 * @todo
 * - Add autoindexing for ID generation
 '/

/'*
 * @addtogroup orxString
 * 
 * String module
 * Module that handles strings
 *
 * @{
 '/


#ifndef _orxSTRING_BI
#define _orxSTRING_BI


#include "orxInclude.bi"
#include "orx/memory/orxMemory.bi"
#include "orx/math/orxVector.bi"

#include "crt/stdio.bi"
#include "crt/stdarg.bi"
#include "crt/string.bi"
#include "crt/stdlib.bi"
#include "crt/ctype.bi"

#include "orx/debug/orxDebug.bi"

#define orxSTRING_KC_VECTOR_START       "("
#define orxSTRING_KC_VECTOR_START_ALT   "{"
#define orxSTRING_KC_VECTOR_SEPARATOR   ","
#define orxSTRING_KC_VECTOR_END         ")"
#define orxSTRING_KC_VECTOR_END_ALT     "}"

Extern "Windows"

/'* Continues a CRC with a string one
 * @param[in] _zString        String used to continue the given CRC
 * @param[in] _u32CRC         Base CRC.
 * @return The resulting CRC.
 '/
Declare Function orxString_ContinueCRC(ByVal _zString As Const orxSTRING, ByVal _u32CRC As orxU32) As orxU32

/'* Continues a CRC with a string one
 * @param[in] _zString        String used to continue the given CRC
 * @param[in] _u32CRC         Base CRC.
 * @param[in] _u32CharNumber  Number of character to process
 * @return The resulting CRC.
 '/
Declare Function orxString_NContinueCRC(ByVal _zString As Const orxSTRING, ByVal _u32CRC As orxU32, ByVal _u32CharNumber As orxU32) As orxU32

End Extern

/' *** String inlined functions *** '/


/'* Skips all white spaces
 * @param[in] _zString        Concerned string
 * @return    Sub string located after all leading white spaces
 '/
#macro orxString_SkipWhiteSpaces(_zString,_result)


  /' Non null? '/
  If(_zString <> orxNULL) Then
  
    /' Skips all white spaces '/
    _result = _zString
		'Asc(" "") = 32:Asc(!"\t") = 9
		While (_result[0][0] = 32 OrElse _result[0][0] = 9)
			/' Empty? '/
			If(*_result = orxCHAR_NULL) Then			
				/' Updates result '/
				_result = orxSTRING_EMPTY
			End If
			_result += 1
		Wend  
  Else  
    /' Updates result '/
    _result = orxNULL
  End If  
  /' Done! '/
  
#endmacro

/'* Returns the number of character in the string
 * @param[in] _zString  String used for length computation
 * @return Length of the string (doesn't count final orxCHAR_NULL)
 '/
#macro orxString_GetLength(_zString,_result)

  /' Checks '/
  orxASSERT(_zString <> orxNULL)

  /' Done! '/
  _result = Cast(orxU32,Len(*_zString))

#endmacro

/'* Copies N characters from a string
 * @param[in] _zDstString       Destination string
 * @param[in] _zSrcString       Source string
 * @param[in] _u32CharNumber    Number of characters to copy
 * @return Copied string
 '/
#macro orxString_NCopy(_zDstString,_zSrcString,_u32CharNumber,_result)

  /' Checks '/
  orxASSERT(_zDstString <> orxNULL)
  orxASSERT(_zSrcString <> orxNULL)

  /' Done! '/
  _result = strncpy(_zDstString, _zSrcString, _u32CharNumber)

#endmacro

/'* Copies a string.
 * @param[in] _zDstString      Destination string
 * @param[in] _zSrcString      Source string
 * @return Copied string.
 '/
#macro orxString_Copy(_zDstString,_zSrcString,_result)

  /' Checks '/
  orxASSERT(_zDstString <> orxNULL)
  orxASSERT(_zSrcString <> orxNULL)

  /' Done! '/
  _result = strcpy(_zDstString, _zSrcString)

#endmacro

/'* Duplicate a string.
 * @param[in] _zSrcString  String to duplicate.
 * @return Duplicated string.
 '/
#macro orxString_Duplicate(_zSrcString,_result)

Scope
		Dim As orxU32  u32Size		

		/' Checks '/
		orxASSERT(_zSrcString <> orxNULL)

		/' Gets string size in bytes '/
		u32Size = (orxString_GetLength(_zSrcString) + 1) * SizeOf(orxCHAR)

		/' Allocates it '/
		_result  = Cast(orxSTRING,orxMemory_Allocate(u32Size, orxMEMORY_TYPE_TEXT))

		/' Valid? '/
		If(zResult <> orxNULL) Then  
			/' Copies source to it '/
			orxMemory_Copy(zResult, _zSrcString, u32Size)
		End If

		/' Done! '/
End Scope  
#endmacro

/'* Deletes a string
 * @param[in] _zString                  String to delete
 '/
#macro orxString_Delete(_zString,_result)

  /' Checks '/
  orxASSERT(_zString <> orxNULL)
  orxASSERT(_zString <> orxSTRING_EMPTY)

  /' Frees its memory '/
  orxMemory_Free(_zString)

  /' Done! '/
  _result = orxSTATUS_SUCCESS

#endmacro

/'* Compare two strings. If the first one is smaller than the second, it returns -1,
 * If the second one is bigger than the first, and 0 if they are equals
 * @param[in] _zString1    First String to compare
 * @param[in] _zString2    Second string to compare
 * @return -1, 0 or 1 as indicated in the description.
 '/
#macro orxString_Compare(_zString1,_zString2,_result)

  /' Checks '/
  orxASSERT(_zString1 <> orxNULL)
  orxASSERT(_zString2 <> orxNULL)

  /' Done! '/
  _result = strcmp(_zString1, _zString2)

#endmacro

/'* Compare N first character from two strings. If the first one is smaller
 * than the second, it returns -1, If the second one is bigger than the first,
 * and 0 if they are equals.
 * @param[in] _zString1       First String to compare
 * @param[in] _zString2       Second string to compare
 * @param[in] _u32CharNumber  Number of character to compare
 * @return -1, 0 or 1 as indicated in the description.
 '/
#macro orxString_NCompare(_zString1,_zString2,_u32CharNumber,_result)

  /' Checks '/
  orxASSERT(_zString1 <> orxNULL)
  orxASSERT(_zString2 <> orxNULL)

  /' Done! '/
  _result = (_zString1, _zString2, _u32CharNumber)

#endmacro

/'* Converts a String to a signed int value using the given base
 * @param[in]   _zString        String To convert
 * @param[in]   _u32Base        Base of the read value (generally 10, but can be 16 to read hexa)
 * @param[out]  _ps32OutValue   Converted value
 * @param[out]  _pzRemaining    If non null, will contain the remaining string after the number conversion
 * @return  orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
#macro orxString_ToS32Base(_zString,_u32Base,_ps32OutValue,_pzRemaining,_result)

Scope
		Dim As orxCHAR Ptr pcEnd

		/' Checks '/
		orxASSERT(_ps32OutValue <> orxNULL)
		orxASSERT(_zString <> orxNULL)

		/' Convert '/
		*_ps32OutValue = strtol(_zString,@pcEnd, _u32Base)
		
		/' Valid conversion ? '/
		If((pcEnd <> _zString) && (_zString[0] <> orxCHAR_NULL)) Then
		
			/' Updates result '/
			_result = orxSTATUS_SUCCESS;		
		Else		
			/' Updates result '/
			_result = orxSTATUS_FAILURE;		
		End If

		/' Asks for remaining string? '/
		If(_pzRemaining <> orxNULL) Then
			/' Stores it '/
			*_pzRemaining = pcEnd
		}

		/' Done! '/
End Scope	
#endmacro

/'* Converts a String to a signed int value, guessing the base
 * @param[in]   _zString        String To convert
 * @param[out]  _ps32OutValue   Converted value
 * @param[out]  _pzRemaining    If non null, will contain the remaining string after the number conversion
 * @return  orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
#macro orxString_ToS32(_zString,_ps32OutValue,_pzRemaining,_result)

  /' Checks '/
  orxASSERT(_ps32OutValue <> orxNULL)
  orxASSERT(_zString <> orxNULL)

  /' Hexadecimal? '/
  If((_zString[0][0] <> orxCHAR_EOL) AndAlso _
    (_zString[0][0] = 0) AndAlso _
    (_zString[0][1] <> orxCHAR_EOL) AndAlso _
    ((_zString[0][1] Or &H20) = 120)) Then
  
    /' Gets hexa value '/
    orxString_ToS32Base(_zString + 2, 16, _ps32OutValue, _pzRemaining,_result)
  
  /' Binary? '/
  ElseIf((_zString[0] <> orxCHAR_EOL) AndAlso _
       (_zString[0] = 0) AndAlso _
       (_zString[0][1] <> orxCHAR_EOL) AndAlso _
       ((_zString[0][1] Or &H20) = 98)) Then
  
    /' Gets binary value '/
    orxString_ToS32Base(_zString + 2, 2, _ps32OutValue, _pzRemaining,_result)  
  /' Octal? '/
  ElseIf((_zString[0][0] <> orxCHAR_EOL) AndAlso _
  ((_zString[0][0] Or &H20) = 0)) Then
  
    /' Gets octal value '/
    orxString_ToS32Base(_zString + 1, 8, _ps32OutValue, _pzRemaining,_result)
  
  /' Decimal '/
  Else  
    /' Gets decimal value '/
    orxString_ToS32Base(_zString, 10, _ps32OutValue, _pzRemaining,_result)
  End If

  /' Done! '/
  
#endmacro

/'* Converts a String to an unsigned int value using the given base
 * @param[in]   _zString        String To convert
 * @param[in]   _u32Base        Base of the read value (generally 10, but can be 16 to read hexa)
 * @param[out]  _pu32OutValue   Converted value
 * @param[out]  _pzRemaining    If non null, will contain the remaining string after the number conversion
 * @return  orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
#macro orxString_ToU32Base(_zString,_u32Base,_pu32OutValue,_pzRemaining,_result)

Scope
		Dim As orxCHAR Ptr pcEnd
		

		/' Checks '/
		orxASSERT(_pu32OutValue <> orxNULL)
		orxASSERT(_zString <> orxNULL)

		/' Convert '/
		*_pu32OutValue = strtoul(_zString, @pcEnd, _u32Base)
		
		/' Valid conversion ? '/
		If((pcEnd <> _zString) AndAlso (_zString[0][0] <> orxCHAR_NULL)) Then		
			/' Updates result '/
			_result = orxSTATUS_SUCCESS;		
		Else		
			/' Updates result '/
			_result  = orxSTATUS_FAILURE
		End If

		/' Asks for remaining string? '/
		If(_pzRemaining <> orxNULL) Then		
			/' Stores it '/
			*_pzRemaining = pcEnd
		End If

		/' Done! '/	
		
End Scope

#endmacro

/'* Converts a String to an unsigned int value, guessing the base
 * @param[in]   _zString        String To convert
 * @param[out]  _pu32OutValue   Converted value
 * @param[out]  _pzRemaining    If non null, will contain the remaining string after the number conversion
 * @return  orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
#macro orxString_ToU32(_zString,_pu32OutValue,_pzRemaining,_result)  

  /' Checks '/
  orxASSERT(_pu32OutValue <> orxNULL)
  orxASSERT(_zString <> orxNULL)

  /' Hexadecimal? '/
  If((_zString[0][0] <> orxCHAR_EOL) AndAlso _
   (_zString[0][0] = 0) AndAlso _
   (_zString[0][1] <> orxCHAR_EOL) AndAlso _
   ((_zString[0][1] Or &H20) = 120)) Then
  
    /' Gets hexa value '/
   orxString_ToU32Base(_zString + 2, 16, _pu32OutValue, _pzRemaining,_result)
  
  /' Binary? '/
  ElseIf((_zString[0][0] <> orxCHAR_EOL) AndAlso _
       (_zString[0][0] = 0) AndAlso _
       (_zString[0][1] <> orxCHAR_EOL) AndAlso _
       ((_zString[0][1] Or &H20) = 98)) Then
  
    /' Gets binary value '/
   orxString_ToU32Base(_zString + 2, 2, _pu32OutValue, _pzRemaining,_result)
  /' Octal? '/
  ElseIf((_zString[0][0] <> orxCHAR_EOL) AndAlso _
     ((_zString[0][0] Or &H20) = 48)) Then  
    /' Gets octal value '/
    orxString_ToU32Base(_zString + 1, 8, _pu32OutValue, _pzRemaining,_result)
  /' Decimal '/
  Else  
    /' Gets decimal value '/
     orxString_ToU32Base(_zString, 10, _pu32OutValue, _pzRemaining,_result)
  End If

  /' Done! '/
  
#endmacro

/'* Convert a string to a value
 * @param[in]   _zString        String To convert
 * @param[out]  _pfOutValue     Converted value
 * @param[out]  _pzRemaining    If non null, will contain the remaining string after the number conversion
 * @return  orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
#macro orxString_ToFloat(_zString,_pfOutValue,_pzRemaining,_result)

Scope
	 Dim As orxCHAR Ptr pcEnd
   

		/' Checks '/
		orxASSERT(_pfOutValue <> orxNULL)
		orxASSERT(_zString <> orxNULL)

		/' Linux / Mac / GP2X / Wii / IPhone / MSVC? '/
	#if defined(__orxLINUX__) Or defined(__orxMAC__) Or defined(__orxMSVC__)

		/' Converts it '/
		*_pfOutValue = Cast(orxFLOAT,strtod(_zString, @pcEnd))

	#else /' __orxLINUX__ || __orxMAC__ || __orxGP2X__ || __orxWII__ || __orxIPHONE__ || __orxMSVC__ '/

		/' Converts it '/
		*_pfOutValue = strtof(_zString, @pcEnd)

	#endif /' __orxLINUX__ || __orxMAC__ || __orxGP2X__ || __orxWII__ || __orxIPHONE__ || __orxMSVC__ '/

		/' Valid conversion ? '/
		If((pcEnd <> _zString) AndAlso (_zString[0][0] <> orxCHAR_NULL)) Then
				/' Updates result '/
			_result = orxSTATUS_SUCCESS
		Else  
			/' Updates result '/
			_result = orxSTATUS_FAILURE
		End If

		/' Asks for remaining string? '/
		If(_pzRemaining <> orxNULL) Then
		
			/' Stores it '/
			*_pzRemaining = pcEnd
		End If

		/' Done! '/
End Scope

#endmacro

/'* Convert a string to a vector
 * @param[in]   _zString        String To convert
 * @param[out]  _pvOutValue     Converted value
 * @param[out]  _pzRemaining    If non null, will contain the remaining string after the number conversion
 * @return  orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
#macro orxString_ToVector(_zString,_pvOutValue,_pzRemaining,_result)

Scope
		Dim As orxVECTOR stValue
		Dim As orxSTRING zString
		

		/' Checks '/
		orxASSERT(_pvOutValue <> orxNULL)
		orxASSERT(_zString <> orxNULL)

		/' Skips all white spaces '/
		orxString_SkipWhiteSpaces(_zString,zString)

		/' Is a vector start character? '/
		If((*zString = orxSTRING_KC_VECTOR_START) OrElse _
		 (*zString = orxSTRING_KC_VECTOR_START_ALT))  
			/' Skips all white spaces '/
			orxString_SkipWhiteSpaces(zString + 1,zString)
			/' Gets X value '/
			orxString_ToFloat(zString, @(stValue.fX), @zString,_result)
			/' Success? '/
			If(_result <> orxSTATUS_FAILURE) Then    
				/' Skips all white spaces '/
				orxString_SkipWhiteSpaces(zString,zString)
				/' Is a vector separator character? '/
				If(*zString = orxSTRING_KC_VECTOR_SEPARATOR) Then      
					/' Skips all white spaces '/
					orxString_SkipWhiteSpaces(zString + 1,zString)
					/' Gets Y value '/
					orxString_ToFloat(zString, @(stValue.fY), @zString,_result)
					/' Success? '/
					If(_result <> orxSTATUS_FAILURE) Then        
						/' Skips all white spaces '/
						orxString_SkipWhiteSpaces(zString,zString)
						/' Is a vector separator character? '/
						If(*zString = orxSTRING_KC_VECTOR_SEPARATOR) Then          
							/' Skips all white spaces '/
							orxString_SkipWhiteSpaces(zString + 1,zString)
							/' Gets Z value '/
							orxString_ToFloat(zString, @(stValue.fZ), @zString,_result)
							/' Success? '/
							If(_result <> orxSTATUS_FAILURE) Then            
								/' Skips all white spaces '/
								orxString_SkipWhiteSpaces(zString,zString)
								/' Is not a vector end character? '/
								If((*zString <> orxSTRING_KC_VECTOR_END) AndAlso
								 (*zString <> orxSTRING_KC_VECTOR_END_ALT)) Then              
									/' Updates result '/
									_result = orxSTATUS_FAILURE
								End If
							End If
						End If
					End If
				End If
			End If
		End If

		/' Valid? '/
		If(_result <> orxSTATUS_FAILURE) Then  
			/' Updates vector '/
			Dim As orxVECTOR vTEMP
			orxVector_Copy(_pvOutValue, @stValue,vTEMP)
			/' Asks for remaining string? '/
			If(_pzRemaining <> orxNULL) Then			
				/' Stores it '/
				*_pzRemaining = zString + 1
			End If
		End If

		/' Done! '/
End Scope

#endmacro

/'* Convert a string to a boolean
 * @param[in]   _zString        String To convert
 * @param[out]  _pbOutValue     Converted value
 * @param[out]  _pzRemaining    If non null, will contain the remaining string after the number conversion
 * @return  orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
#macro orxString_ToBool(_zString,_pbOutValue,_pzRemaining,_result)

Scope
		Dim As orxS32 s32Value
		Dim As orxS32 s32Res

		/' Checks '/
		orxASSERT(_pbOutValue <> orxNULL)
		orxASSERT(_zString <> orxNULL)

		/' Tries numeric value '/
		orxString_ToS32Base(_zString, 10, &s32Value, _pzRemaining,_result)

		/' Valid? '/
		If(_result <> orxSTATUS_FAILURE) Then		
			/' Updates boolean '/
			*_pbOutValue = Iif(s32Value <> 0,orxTRUE,orxFALSE)		
		Else		
			Dim As orxU32 u32Length
			/' Gets length of false '/
			orxString_GetLength(orxSTRING_FALSE,u32Length)
			/' Is false? '/
			orxString_NCompare(_zString, orxSTRING_FALSE, u32Length,s32Res)
			If(s32Res = 0) Then
			/' Updates boolean '/
				*_pbOutValue = orxFALSE
				/' Has remaining? '/
				If(_pzRemaining <> orxNULL) Then				
					/' Updates it '/
					*_pzRemaining += u32Length
				End If
				/' Updates result '/
				_result  = orxSTATUS_SUCCESS			
			Else			
				/' Gets length of true '/
				orxString_GetLength(orxSTRING_TRUE,u32Length)
				/' Is true? '/				
				s32Res = orxString_NCompare(_zString, orxSTRING_TRUE, u32Length)
				If( s32Res = 0) Then				
					/' Updates boolean '/
					*_pbOutValue = orxTRUE
					/' Has remaining? '/
					If(_pzRemaining <> orxNULL) Then					
						/' Updates it '/
						*_pzRemaining += u32Length
					End If
					/' Updates result '/
					_result  = orxSTATUS_SUCCESS
				End If
			End If
		End If
		/' Done! '/		
End Scope
#endmacro


/'* Lowercase a string
 * @param[in] _zString          String To convert
 * @return The converted string.
 '/
#macro orxString_LowerCase(_zString,_result)

Scope
		Dim As orxCHAR Ptr pc

		/' Checks '/
		orxASSERT(_zString <> orxNULL)

		/' Converts the whole string '/
		pc = _zString
		While(*pc <> orxCHAR_NULL)		
			/' Needs to be converted? '/
			If(pc[0][0] >= 65 AndAlso pc[0][0] <= 90) Then    
				/' Lower case '/
				pc[0][0] Or= &H20;    
			End If
			pc += 1
		Wend
		_result = _zString
End Scope
#endmacro

/'* Uppercase a string
 * @param[in] _zString          String To convert
 * @return The converted string.
 '/
#macro orxString_UpperCase(_zString,_result)

Scope
		Dim As orxCHAR Ptr pc
		/' Checks '/
		orxASSERT(_zString <> orxNULL)
		/' Converts the whole string '/
		pc = _zString
		While (*pc <> orxCHAR_NULL)  
			/' Needs to be converted? '/
			If (pc[0][0] >= 97 && pc[0][0] <= 122) Then    
				/' Upper case '/
				pc[0][0] And= Not(&H20)
			End If
			pc += 1
		Wend
		_result = _zString
End Scope
#endmacro

/'* Converts a string to a CRC
 * @param[in] _zString          String To convert
 * @return The resulting CRC.
 '/
#macro orxString_ToCRC(_zString,_result)

  /' Checks '/
  orxASSERT(_zString <> orxNULL)

  /' Computes the ID '/
  _result = orxString_ContinueCRC(_zString, 0))

#endmacro

/'* Converts a string to a CRC
 * @param[in] _zString        String To convert
 * @param[in] _u32CharNumber  Number of characters to process
 * @return The resulting CRC.
 '/
#macro orxString_NToCRC(_zString,_u32CharNumber,_result)

  /' Checks '/
  orxASSERT(_zString <> orxNULL)

  /' Computes the ID '/
  _result = orxString_NContinueCRC(_zString, 0, _u32CharNumber)

#endmacro

/'* Returns the first occurence of _zString2 in _zString1
 * @param[in] _zString1 String to analyze
 * @param[in] _zString2 String that must be inside _zString1
 * @return The pointer of the first occurence of _zString2, or orxNULL if not found
 '/
#macro orxString_SearchString(_zString1,_zString2,_result)

  /' Checks '/
  orxASSERT(_zString1 <> orxNULL)
  orxASSERT(_zString2 <> orxNULL)

  /' Returns result '/
  _result = strstr(_zString1, _zString2)

#endmacro

/'* Returns the first occurence of _cChar in _zString
 * @param[in] _zString String to analyze
 * @param[in] _cChar   The character to find
 * @return The pointer of the first occurence of _cChar, or orxNULL if not found
 '/
#macro orxString_SearchChar(_zString,_cChar,_result)

  /' Checks '/
  orxASSERT(_zString <> orxNULL)

  /' Returns result '/
  _result = strchr(_zString, _cChar)

#endmacro

/'* Returns the first occurence of _cChar in _zString
 * @param[in] _zString      String to analyze
 * @param[in] _cChar        The character to find
 * @param[in] _u32Position  Search begin position
 * @return The index of the next occurence of requested character, starting at given position / -1 if not found
 '/
#macro orxString_SearchCharIndex(_zString,_cChar,_u32Position,_result)

Scope
		
		_result = -1
		Dim As orxS32    s32Index
		Dim As orxCHAR  Ptr pc
    Dim As orxU32 uTEMP
		/' Checks '/
		orxASSERT(_zString <> orxNULL)
		
		orxString_GetLength(_zString,uTEMP)
		orxASSERT(_u32Position <= uTEMP)

		/' For all characters '/
		s32Index = _u32Position:pc = _zString + s32Index
		While(*pc <> orxCHAR_NULL)  
			/' Found? '/
			If(pc[0][0] = _cChar) Then    
				/' Updates result '/
				s32Result = s32Index
				Exit While
			End If
			pc += 1
			s32Index += 1
		Wend

		/' Done! '/
		_result = s32Result
End Scope
#endmacro

'/'* Prints a formated string to a memory buffer
 '* @param[out] _zDstString  Destination string
 '* @param[in]  _zSrcString  Source formated string
 '* @return The number of written characters
 ''/
'static orxINLINE orxS32 orxCDECL        orxString_Print(orxSTRING _zDstString, orxSTRING _zSrcString, ...)
'{
  'va_list stArgs
  'orxS32  s32Result

  '/' Checks '/
  'orxASSERT(_zDstString <> orxNULL)
  'orxASSERT(_zSrcString <> orxNULL)

  '/' Gets variable arguments & print the string '/
  'va_start(stArgs, _zSrcString)
  's32Result = vsprintf(_zDstString, _zSrcString, stArgs)
  'va_end(stArgs)

  '/' Done! '/
  'return s32Result
'}

'/'* Prints a formated string to a memory buffer using a max size
 '* @param[out] _zDstString    Destination string
 '* @param[in]  _zSrcString    Source formated string
 '* @param[in]  _u32CharNumber Max number of character to print
 '* @return The number of written characters
 ''/
'static orxINLINE orxS32 orxCDECL        orxString_NPrint(orxSTRING _zDstString, orxU32 _u32CharNumber, orxSTRING _zSrcString, ...)
'{
  'va_list stArgs
  'orxS32  s32Result

  '/' Checks '/
  'orxASSERT(_zDstString <> orxNULL)
  'orxASSERT(_zSrcString <> orxNULL)

  '/' Gets variable arguments & print the string '/
  'va_start(stArgs, _zSrcString)
  's32Result = vsnprintf(_zDstString, _u32CharNumber, _zSrcString, stArgs)
  'va_end(stArgs)

  '/' Done! '/
  'return s32Result
'}



'Declare Function orxString_NPrint cdecl (ByVal _zDstString As orxSTRING, ByVal _u32CharNumber As orxU32, ByVal _zSrcString As orxSTRING, ...) As orxS32

#endif /' _orxSTRING_BI '/

/'* @} '/
